
/*************************************************************************

  APU R 2.0

  intmodp_bigint.c

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  
#include "bigint.h"
#include "intmodp.h"
#include "intmodp_bigint.h"

/* BigInt's are first converted to mod p numbers and the result is
   returned as IntModp's */

IntModp 
operator+ (const IntModp& a, const BigInt& b) 
{
  IntModp result;
  int bval; 
  if ((b.i_value()).islong()) 
    bval = (int)(b.i_value()).tolong(); 
  else { 
    bval = (int)(b.i_value() % integer(a.prime_of())).tolong(); 
  }
  int temp = (int)
  (fmod(((double)(a.i_value())+(double)(bval)),(double)(a.prime_of())));
  result = IntModp(temp); 
  return(result);
}

IntModp 
operator+ (const BigInt& a, const IntModp& b) 
{
  IntModp result;
  int aval; 
  if ((a.i_value()).islong()) 
    aval = (int)(a.i_value()).tolong(); 
  else { 
    aval = (int)(a.i_value() % integer(b.prime_of())).tolong(); 
  }
  int temp = (int)
  (fmod(((double)(aval)+(double)(b.i_value())),(double)(b.prime_of())));
  result = IntModp(temp); 
  return(result);
}

IntModp 
operator- (const IntModp& a, const BigInt& b) 
{
  IntModp result;
  int bval; 
  if ((b.i_value()).islong()) 
    bval = (int)(b.i_value()).tolong(); 
  else { 
    bval = (int)(b.i_value() % integer(a.prime_of())).tolong(); 
  }
  int temp = (int)
  (fmod(((double)(a.i_value())-(double)(bval)),(double)(a.prime_of())));
  result = IntModp(temp); 
  return(result);
}

IntModp 
operator- (const BigInt& a, const IntModp& b) 
{
  IntModp result;
  int aval; 
  if ((a.i_value()).islong()) 
    aval = (int)(a.i_value()).tolong(); 
  else { 
    aval = (int)(a.i_value() % integer(b.prime_of())).tolong(); 
  }
  int temp = (int)
  (fmod(((double)(aval)-(double)(b.i_value())),(double)(b.prime_of())));
  result = IntModp(temp); 
  return(result);
}

IntModp 
operator* (const IntModp& a, const BigInt& b) 
{
  IntModp result;  
  int bval; 
  if ((b.i_value()).islong()) 
    bval = (int)(b.i_value()).tolong(); 
  else { 
    bval = (int)(b.i_value() % integer(a.prime_of())).tolong(); 
  }
  long temp; 
  temp = (int)
  (fmod(((double)(a.i_value())*(double)(bval)),(double)(a.prime_of())));
  result = IntModp(temp);
  return(result);
}

IntModp 
operator* (const BigInt& a, const IntModp& b) 
{
  IntModp result;
  int aval; 
  if ((a.i_value()).islong()) 
    aval = (int)(a.i_value()).tolong(); 
  else { 
    aval = (int)(a.i_value() % integer(b.prime_of())).tolong(); 
  }
  long temp; 
  temp = (int)
    (fmod(((double)(aval)*(double)(b.i_value())),(double)(b.prime_of())));
  result = IntModp(temp);
  return(result);
}

IntModp 
operator/ (const IntModp& a, const BigInt& b) 
{
  IntModp result;
  int bval; 
  if ((b.i_value()).islong()) 
    bval = (int)(b.i_value()).tolong(); 
  else { 
    bval = (int)(b.i_value() % integer(a.prime_of())).tolong(); 
  }
  int temp; 
  temp = finitef_inverse(bval,a.prime_of());
  temp = (int) 
   (fmod((double)(temp)*(double)(a.i_value()),(double)(a.prime_of()))); 
  result = IntModp(temp);
  return(result);
}

IntModp 
operator/ (const BigInt& a, const IntModp& b) 
{
  IntModp result;
  int aval; 
  if ((a.i_value()).islong()) 
    aval = (int)(a.i_value()).tolong(); 
  else { 
    aval = (int)(a.i_value() % integer(b.prime_of())).tolong(); 
  }
  int temp; 
  temp = finitef_inverse(b.i_value(),b.prime_of());
  temp = (int)
   (fmod((double)(temp)*(double)(aval),(double)(b.prime_of())));
  result = IntModp(temp);
  return(result);
}

IntModp 
operator% (const IntModp& a, const BigInt& b) 
{
  IntModp result;
  int bval; 
  if ((b.i_value()).islong()) 
    bval = (int)(b.i_value()).tolong(); 
  else { 
    bval = (int)(b.i_value() % integer(a.prime_of())).tolong(); 
  }
  long temp;
  temp = (a.i_value() % bval) % a.prime_of(); 
  result = IntModp(temp);
  return(result);
}

IntModp 
operator% (const BigInt& a, const IntModp& b) 
{
  IntModp result;
  int aval; 
  if ((a.i_value()).islong()) 
    aval = (int)(a.i_value()).tolong(); 
  else { 
    aval = (int)(a.i_value() % integer(b.prime_of())).tolong(); 
  }
  long temp;
  temp = (aval % b.i_value()) % b.prime_of(); 
  result = IntModp(temp);
  return(result);
}

