
/*************************************************************************

  APU R 2.0

  intmodp_smallint.c

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  
#include "smallint.h"
#include "intmodp.h"
#include "intmodp_smallint.h"

IntModp 
operator+ (const IntModp& a, const SmallInt& b) 
{
  IntModp result;
  int temp = (int)
  (fmod(((double)(a.i_value())+(double)(b.i_value())),(double)(a.prime_of())));
  result = IntModp(temp); 
  return(result);
}

IntModp 
operator+ (const SmallInt& a, const IntModp& b) 
{
  IntModp result;
  int temp = (int)
  (fmod(((double)(a.i_value())+(double)(b.i_value())),(double)(b.prime_of())));
  result = IntModp(temp); 
  return(result);
}

IntModp 
operator- (const IntModp& a, const SmallInt& b) 
{
  IntModp result;
  int temp = (int)
  (fmod(((double)(a.i_value())-(double)(b.i_value())),(double)(a.prime_of())));
  result = IntModp(temp); 
  return(result);
}

IntModp 
operator- (const SmallInt& a, const IntModp& b) 
{
  IntModp result;
  int temp = (int)
  (fmod(((double)(a.i_value())-(double)(b.i_value())),(double)(b.prime_of())));
  result = IntModp(temp); 
  return(result);
}

IntModp 
operator* (const IntModp& a, const SmallInt& b) 
{
  IntModp result;  
  long temp; 
  temp = (int)
  (fmod(((double)(a.i_value())*(double)(b.i_value())),(double)(a.prime_of())));
  result = IntModp(temp);
  return(result);
}

IntModp 
operator* (const SmallInt& a, const IntModp& b) 
{
  IntModp result;
  long temp; 
  temp = (int)
    (fmod(((double)(a.i_value())*(double)(b.i_value())),(double)(b.prime_of())));
  result = IntModp(temp);
  return(result);
}

IntModp 
operator/ (const IntModp& a, const SmallInt& b) 
{
  IntModp result;
  int temp; 
  temp = finitef_inverse(b.i_value(),a.prime_of());
  result = IntModp(temp*a.i_value());
  return(result);
}

IntModp 
operator/ (const SmallInt& a, const IntModp& b) 
{
  IntModp result;
  int temp; 
  temp = finitef_inverse(b.i_value(),b.prime_of());
  result = IntModp(temp*a.i_value());
  return(result);
}

IntModp 
operator% (const IntModp& a, const SmallInt& b) 
{
  IntModp result;
  long temp;
  temp = a.i_value() % b.i_value(); 
  result = IntModp(temp);
  return(result);
}

IntModp 
operator% (const SmallInt& a, const IntModp& b) 
{
  IntModp result;
  long temp;
  temp = a.i_value() % b.i_value(); 
  result = IntModp(temp);
  return(result);
}

