/*************************************************************************

  APU R 2.0

  bigfloatpt.h

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  

#if !defined(BIGFLOATHDR)
#define BIGFLOATHDR

#include "globals.h"

#include "fielddef.h"
#include "bigfloat.h"

class BigFloat : public Field_element {
  private : 
    static const char* Class_name; 
  public : 

    BigFloat(); 
    BigFloat(int);
    BigFloat(long); 
    BigFloat(const integer&); // LEDA "integer" type
    BigFloat(double);
    BigFloat(const bigfloat&); // LEDA "bigfloat" type
    const char* class_name(); 

    friend BigFloat operator + (const BigFloat&, const BigFloat&);
    friend BigFloat operator - (const BigFloat&, const BigFloat&);
    friend BigFloat operator * (const BigFloat&, const BigFloat&);
    friend BigFloat operator / (const BigFloat&, const BigFloat&);

    friend BigFloat operator ^ (const BigFloat&, int);
    friend BigFloat operator - (const BigFloat&); // negation

    friend bool operator == (const BigFloat&, const BigFloat&);
    friend bool operator != (const BigFloat&, const BigFloat&);
    friend bool operator  < (const BigFloat&, const BigFloat&);
    friend bool operator <= (const BigFloat&, const BigFloat&);
    friend bool operator  > (const BigFloat&, const BigFloat&);
    friend bool operator >= (const BigFloat&, const BigFloat&);

    friend bool is_zero (const BigFloat&);
    friend int sign(const BigFloat&);
}; 


inline BigFloat::BigFloat() {
  type = BIGFLOAT;
  frep = (void*) (new bigfloat); 
  *((bigfloat*)frep) = bigfloat(0); 
}

inline BigFloat::BigFloat(int a = 0) {
  type = BIGFLOAT;
  frep = (void*) (new bigfloat); 
  *((bigfloat*)frep) = bigfloat(a); 
}

inline BigFloat::BigFloat(long a = 0) {
  type = BIGFLOAT;
  frep = (void*) (new bigfloat); 
  *((bigfloat*)frep) = bigfloat(a); 
}

inline BigFloat::BigFloat(const integer& a) {
  type = BIGFLOAT; 
  frep = (void*)(new bigfloat); 
  *((bigfloat*)frep) = bigfloat(a); 
}

inline BigFloat::BigFloat(double a = 0) {
  type = BIGFLOAT;
  frep = (void*) (new bigfloat); 
  *((bigfloat*)frep) = bigfloat(a); 
}

inline BigFloat::BigFloat(const bigfloat& a) {
  type = BIGFLOAT; 
  frep = (void*)(new bigfloat); 
  *((bigfloat*)frep) = a; 
}

inline const char* BigFloat::class_name() {
  return(Class_name); 
}

#endif
