/*************************************************************************

  APU R 2.0

  float.h

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  

#if !defined(FLOAT_HDR)
#define FLOAT_HDR

#include "globals.h"

#include "fielddef.h"

class Float : public Field_element {
  private : 
    static const char* Class_name; 
  public : 

    Float(int);
    Float(float); 
    Float(double); 
    const char* class_name(); 
    double f_value() const;
    void print() const;

    friend Float operator + (const Float&, const Float&);
    friend Float operator - (const Float&, const Float&);
    friend Float operator * (const Float&, const Float&);
    friend Float operator / (const Float&, const Float&);

    friend Float operator % (const Float&, const Float&);
    friend Float operator ^ (const Float&, const Float&);
    friend Float operator ^ (const Float&, int);

    friend Float operator - (const Float&); // negation

    friend bool operator == (const Float&, const Float&);
    friend bool operator != (const Float&, const Float&);
    friend bool operator  < (const Float&, const Float&);
    friend bool operator <= (const Float&, const Float&);
    friend bool operator  > (const Float&, const Float&);
    friend bool operator >= (const Float&, const Float&);

    friend bool is_zero (const Float&);
    friend int sign(const Float&);
}; 


inline Float::Float(int a = 0) {
  type = FLOAT;
  frep = (void*) (new double); 
  *((double*)frep) = (double)(a); 
}

inline Float::Float(float a) {
  type = FLOAT; 
  frep = (void*)(new double); 
  *((double*)frep) = (double)(a); 
}

inline Float::Float(double a) {
  type = FLOAT; 
  frep = (void*)(new double); 
  *((double*)frep) = a; 
}

inline const char* Float::class_name() {
  return(Class_name); 
}

inline double Float::f_value() const { 
  return(*((double*)frep));
}

inline void Float::print() const {
  cout << this->f_value();
}

#endif
