/*************************************************************************

  APU R 2.0

  bigrat.h

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  

#if !defined(BIGRAT_HDR)
#define BIGRAT_HDR

#include "globals.h"

#include "fielddef.h"
#include "integer.h"
#include "rational.h"

class BigRat : public Field_element {
  private : 
    static const char* Class_name; 
  public : 

    BigRat(); 
    BigRat(int);
    BigRat(int,int);
    BigRat(long); 
    BigRat(const integer&); // LEDA "integer" type
    BigRat(const integer&, const integer&); // LEDA "integer" type
    BigRat(const rational&); // LEDA "rational" type 
    const char* class_name(); 
    integer numer_of() const; 
    integer denom_of() const; 
    rational& rational_of() const; 
    void print();

    friend BigRat operator + (const BigRat&, const BigRat&);
    friend BigRat operator - (const BigRat&, const BigRat&);
    friend BigRat operator * (const BigRat&, const BigRat&);
    friend BigRat operator / (const BigRat&, const BigRat&);

    friend BigRat operator % (const BigRat&, const BigRat&);
    friend BigRat operator ^ (const BigRat&, int);

    friend BigRat operator - (const BigRat&); // negation

    friend bool operator == (const BigRat&, const BigRat&);
    friend bool operator != (const BigRat&, const BigRat&);
    friend bool operator  < (const BigRat&, const BigRat&);
    friend bool operator <= (const BigRat&, const BigRat&);
    friend bool operator  > (const BigRat&, const BigRat&);
    friend bool operator >= (const BigRat&, const BigRat&);

    friend ostream& operator << (ostream&, const BigRat&);  
    friend bool is_zero(const BigRat&);
    friend int sign(const BigRat&);
}; 

inline rational& BigRat::rational_of() const { 
  return(*((rational*)frep)); 
}

inline BigRat::BigRat() {
  type = BIGRAT;
  frep = (void*) (new rational); 
  *((rational*)frep) = rational(0); 
}

inline BigRat::BigRat(int a) {
  type = BIGRAT;
  frep = (void*) (new rational); 
  *((rational*)frep) = rational(a); 
}

inline BigRat::BigRat(int a,int b) {
  type = BIGRAT;
  frep = (void*) (new rational); 
  *((rational*)frep) = rational(a,b); 
}

inline BigRat::BigRat(long a) {
  type = BIGRAT;
  frep = (void*) (new rational); 
  *((rational*)frep) = rational((int)a); 
}

inline BigRat::BigRat(const integer& a) {
  type = BIGRAT;
  frep = (void*) (new rational); 
  *((rational*)frep) = rational(a); 
}

inline BigRat::BigRat(const integer& a, const integer& b) {
  type = BIGRAT; 
  frep = (void*) (new rational); 
  *((rational*)frep) = rational(a,b); 
}

inline BigRat::BigRat(const rational& a) {
  type = BIGRAT;
  frep = (void*) (new rational); 
  *((rational*)frep) = a; 
}

inline const char* BigRat::class_name() {
  return(Class_name); 
}

inline integer BigRat::numer_of() const {
  return(((rational*)frep)->numerator()); 
}

inline integer BigRat::denom_of() const {
  return(((rational*)frep)->denominator()); 
}

inline integer numer_of(const BigRat& a) {
  return(a.numer_of()); 
}

inline integer denom_of(const BigRat& a) {
  return(a.denom_of()); 
}

inline void BigRat::print() {
  cout << "<";
  cout << *((rational*)frep); 
  cout << ">";
}

inline
ostream& operator << (ostream& os, const BigRat& a) {
  os << "<"; 
  os << *((rational*)a.frep); 
  os << ">";
}

inline 
rational& normalize (rational a) { 
  return(a.normalize()); 
}


#endif
