/*************************************************************************

  APU R 2.0

  intmodp.h

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  

#if !defined(INTMODPHDR)
#define INTMODPHDR

#include "globals.h"

#include "fielddef.h"

class IntModp : public Field_element {
  friend class Field_element;
  private : 
    static const char* Class_name; 
  public : 
    static int prime; 

    IntModp(int);
    int i_value() const;
    int prime_of() const; 
    void print() ; 
    const char* class_name(); 

    friend IntModp operator + (const IntModp&, const IntModp&);
    friend IntModp operator - (const IntModp&, const IntModp&);
    friend IntModp operator * (const IntModp&, const IntModp&);
    friend IntModp operator / (const IntModp&, const IntModp&);
    friend IntModp operator % (const IntModp&, const IntModp&);

    friend IntModp operator ^ (const IntModp&, const IntModp&);
    friend IntModp operator ^ (const IntModp&, int);
    friend IntModp operator ^ (const IntModp&, long int);

    friend IntModp operator - (const IntModp&); // negation

    friend bool operator == (const IntModp&, const IntModp&);
    friend bool operator != (const IntModp&, const IntModp&);
    friend bool is_zero (const IntModp&);
}; 

__inline__ int IntModp::prime_of() const {
  return(prime); 
}

__inline__ IntModp::IntModp(int a = 0) {
  type = INTMODP;
  int temp = (int)(a % prime); 
  if (temp < 0) temp = temp + prime; 
  frep = (void *) temp;
}

inline int IntModp::i_value() const {
  return((int)(frep));
}

inline void IntModp::print() { 
  cout << this.i_value() ;
}

inline const char* IntModp::class_name() {
  return(Class_name); 
}

/*
inline
IntModp 
operator* (const IntModp& a, const IntModp& b) 
{
  IntModp result;
  long temp; 
  temp = (a.i_value()*b.i_value()) % a.prime_of(); 
  result = IntModp(temp);
  return(result);
}
*/

extern int finitef_inverse(int, int);
extern int finitef_mult(int,int,int); 
extern int finitef_add(int,int,int); 
extern int finitef_sub(int,int,int); 

extern "C" unsigned modmult(unsigned a, unsigned b, unsigned prime); 
#endif
