
/*************************************************************************

  APU R 2.0

  matrix.h

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  

#if !defined(MATRIX_HDR) 
#define MATRIX_HDR

#include <iostream.h>
#include "array.h"


template <class Type>
class Matrix {
  private : 
    Array<Type> *matrix;
    int dim1,dim2;
  public : 
    Matrix() : matrix(0), dim1(0), dim2(0) {}; 
    Matrix(int m, int n) : matrix(new Array<Type>[m]), dim1(m), dim2(n)
                  { for (int i=0;i<m;i++) matrix[i] = Array<Type>(n); }

    Array<Type>& operator[](int ix) { return matrix[ix]; } 
    Array<Type>& operator[](int ix) const { return matrix[ix]; } 
    int dim1_of() const { return dim1; } 
    int dim2_of() const { return dim2; }
    void set_dim1(int val) { dim1 = val; } 
    void set_dim2(int val) { dim2 = val; } 
    bool is_empty() const 
      {if ((matrix == 0) || (dim1 == 0)) return(true); else return(false);}
    friend Array<Type> operator * (const Matrix<Type>&, const Array<Type>&); 
    friend ostream& operator << (ostream&, const Matrix<Type>&); 
};

template <class Type>
inline int dim1_of(const Matrix<Type>& mat) {
  return(mat.dim1_of());
}

template <class Type>
inline int dim2_of(const Matrix<Type>& mat) {
  return(mat.dim2_of());
}

template <class Type> 
inline bool is_empty(const Matrix<Type>& mat) {
  return (mat.is_empty());
}

/* Multiply an m x n matrix with an n x 1 array on the right hand side 
   Result is an m x 1 array 
*/

template <class Type>
Array<Type> operator * (const Matrix<Type>& mat, const Array<Type>& arr) {
  if (dim2_of(mat) != size_of(arr)) {
    cout << "Error in * (Matrix,Array) - dimensions don't match\n";
    return(Array<Type>(0));
  }
  else {
    int m = dim1_of(mat);
    int n = dim2_of(mat);
    Array<Type> result = Array<Type>(m);
    for (int i=0;i<m;i++) {
      result[i] = Type(0);    // init to 0
      for (int j=0;j<n;j++) 
	result[i] = result[i] + mat[i][j] * arr[j];
    }
    return(result);
  }
}

template <class Type>
ostream& operator << (ostream& os, const Matrix<Type>& M)
{
  int dim1 = dim1_of(M); int dim2 = dim2_of(M); 
  for (int i=0;i<dim1;i++) {
    for (int j=0;j<dim2;j++)
      os << M[i][j] << "  ";
    os << "\n";
  }
  return os; 
}

Array<int> lin_solve(Matrix<int>,Array<int>);
Matrix<float> convert_to_float(Matrix<int>);
Matrix<float> matrix_inverse(Matrix<float>);
float** copy_matrix_to_numrec(Matrix<float>, int, int);


#endif
