/*************************************************************************

  APU R 2.0

  multi_intmodp.h

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  

#if !defined(MULTI_INTMODPHDR)
#define MULTI_INTMODPHDR


#include "globals.h"
#include "primelist.h"

#include "fielddef.h"
#include "bigint.h"
#include "integer.h"

#define MULTI_NUMPRIMES EXTRA_ENTRIES

class MultiIntModp : public Field_element {
  friend class Field_element;
  private : 
    static const char* Class_name; 
  public : 
    static int primes[MULTI_NUMPRIMES]; 
    static int CurrentNumPrimes; // CurrentNumPrimes <= MULTI_NUMPRIMES 

    MultiIntModp(int);
    MultiIntModp(int[MULTI_NUMPRIMES]);
    MultiIntModp(BigInt); 
    MultiIntModp(integer); // LEDA (big) integer type
    MultiIntModp(Field_element); 
    int i_value(int) const;
    int prime_of(int) const; 
    void print() ; 
    const char* class_name(); 
    void set_CurrentNumPrimes(int a) const;
    int get_CurrentNumPrimes() const;

    friend MultiIntModp operator + (const MultiIntModp&, const MultiIntModp&);
    friend MultiIntModp operator - (const MultiIntModp&, const MultiIntModp&);
    friend MultiIntModp operator * (const MultiIntModp&, const MultiIntModp&);
    friend MultiIntModp operator / (const MultiIntModp&, const MultiIntModp&);
    friend MultiIntModp operator % (const MultiIntModp&, const MultiIntModp&);

    friend MultiIntModp operator ^ (const MultiIntModp&, const MultiIntModp&);
    friend MultiIntModp operator ^ (const MultiIntModp&, int);
    friend MultiIntModp operator ^ (const MultiIntModp&, long int);

    friend MultiIntModp operator - (const MultiIntModp&); // negation

    friend ostream& operator << (ostream&, const Field_element&); 
    friend bool operator == (const MultiIntModp&, const MultiIntModp&);
    friend bool operator != (const MultiIntModp&, const MultiIntModp&);
    friend bool is_zero (const MultiIntModp&);
    friend int sign(const MultiIntModp&);
}; 

inline int MultiIntModp::prime_of(int i) const {
  return(primes[i]); 
}

inline MultiIntModp::MultiIntModp(int a = 0) {
  type = MULTI_INTMODP;
  for (int i=0;i<CurrentNumPrimes;i++) { 
    int temp = (int)(a % primes[i]); 
    if (temp < 0) temp = temp + primes[i]; 
    frep2[i] = temp;  
  }
}

inline MultiIntModp::MultiIntModp(int a[MULTI_NUMPRIMES]) {
  type = MULTI_INTMODP;
  for (int i=0;i<CurrentNumPrimes;i++) { 
    frep2[i] = a[i];  
  }
}

inline MultiIntModp::MultiIntModp(BigInt a) { 
  type = MULTI_INTMODP; 
  integer temp = a.i_value(); 
  for (int i=0;i<CurrentNumPrimes;i++) { 
    frep2[i] = (int)((temp % integer(primes[i])).tolong()); 
    if (frep2[i] < 0) frep2[i] += primes[i]; 
  }
}


inline MultiIntModp::MultiIntModp(integer a) { 
  type = MULTI_INTMODP; 
  for (int i=0;i<CurrentNumPrimes;i++) { 
    frep2[i] = (int)((a % integer(primes[i])).tolong()); 
    if (frep2[i] < 0) frep2[i] += primes[i]; 
  }
}

inline MultiIntModp::MultiIntModp(Field_element a) { 
  type = MULTI_INTMODP; 
  switch(a.type_of()) { 
    case BIGINT : 
	for (int i=0;i<CurrentNumPrimes;i++) { 
	  integer* temp = (integer*)(a.frep_of());
	  frep2[i] = (int)((*temp % integer(primes[i])).tolong()); 
          if (frep2[i] < 0) frep2[i] += primes[i]; 
        }
	break;
    default: cout << "Unknown type passed to MultiIntModp(Field_element)\n"; 
	break;
  }
}


inline int MultiIntModp::i_value(int i) const {
  return(frep2[i]);
}

inline void MultiIntModp::print() { 
  for (int i=0;i<CurrentNumPrimes;i++) 
    cout << this.i_value(i) << " ";
}

inline ostream& operator << (ostream& os, const MultiIntModp& a) { 
  for (int i=0;i<MultiIntModp::CurrentNumPrimes;i++) 
    os << a.i_value(i) << " ";
}

inline const char* MultiIntModp::class_name() {
  return(Class_name); 
}

inline void MultiIntModp::set_CurrentNumPrimes(int a) const { 
  CurrentNumPrimes = a; 
}

inline int MultiIntModp::get_CurrentNumPrimes() const { 
  return(CurrentNumPrimes);
}

extern int finitef_inverse(int, int);
extern int finitef_mult(int,int,int); 
extern int finitef_add(int,int,int); 
extern int finitef_sub(int,int,int); 
extern "C" unsigned modmult(unsigned,unsigned,unsigned); 

#endif
