/*************************************************************************

  APU R 2.0

  multimod_arith.h

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  

#if !defined(MULTIMOD_ARITHHDR)
#define MULTIMOD_ARITHHDR

#include "multi_intmodp.h"

inline void multimod_make_zeroarray(int* result) {
  for (int i=0;i<MultiIntModp::CurrentNumPrimes;i++) 
    result[i] = 0; 
}

inline void multimod_copy_array(int* to_array, int* from_array)
{
  for (int i=0;i<MultiIntModp::CurrentNumPrimes;i++) 
    to_array[i] = from_array[i];
}

inline void multimod_set_array(int* to_array, int from_val)
{
  if (from_val < 0) {
    for (int i=0;i<MultiIntModp::CurrentNumPrimes;i++) {
      to_array[i] = from_val % MultiIntModp::primes[i];
      to_array[i] += MultiIntModp::primes[i];
    }
  }
  else {
    for (int i=0;i<MultiIntModp::CurrentNumPrimes;i++) 
      to_array[i] = from_val % MultiIntModp::primes[i];
  }
}

inline void multimod_add(int* result, int* a, int* b) 
{
  for (int i=0;i<MultiIntModp::CurrentNumPrimes;i++) { 
    result[i] = (a[i] + b[i]) % MultiIntModp::primes[i];
  }
}

inline void multimod_subtract(int* result, int* a, int* b) 
{
  for (int i=0;i<MultiIntModp::CurrentNumPrimes;i++) { 
    result[i] = (a[i] - b[i]) % MultiIntModp::primes[i];
    if (result[i] < 0) result[i] += MultiIntModp::primes[i];
  }
}

inline void multimod_multiply(int* result, int* a, int* b) 
{
  for (int i=0;i<MultiIntModp::CurrentNumPrimes;i++) { 
    result[i] = (int) modmult((unsigned)a[i],(unsigned)b[i],
			      (unsigned)(MultiIntModp::primes[i]));
  }
}

inline void multimod_divide(int* result, int* a, int* b) 
{
  for (int i=0;i<MultiIntModp::CurrentNumPrimes;i++) { 
    result[i] = (int)finitef_inverse(b[i],MultiIntModp::primes[i]);
    result[i] = (int)modmult((unsigned)a[i],(unsigned)result[i],
			     (unsigned)(MultiIntModp::primes[i]));
  }
}

inline void multimod_exp(int* result,int* a, int b)
{
  for (int i=0;i<MultiIntModp::CurrentNumPrimes;i++) { 
    result[i] = 1;
    for (int j=0;j<b;j++) 
      result[i] = (int) modmult((unsigned)a[i],(unsigned)result[i],
				(unsigned)(MultiIntModp::primes[i])); 
  }
}

inline int is_zeroarray(int* a) { 
  for (int i=0;i<MultiIntModp::CurrentNumPrimes;i++) { 
    if (a[i] != 0) return(0);
  }
  return(1);
}

#endif
