/*************************************************************************

  APU R 2.0

  smallint.h

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  

#if !defined(SMALLINT_HDR)
#define SMALLINT_HDR

#include "globals.h"

#include "fielddef.h"

class SmallInt : public Field_element {
  friend class Field_element;
  private : 
    static const char* Class_name; 
  public : 

    SmallInt(int);
    int i_value() const;
    void print() ; 
    const char* class_name(); 

    friend SmallInt operator + (const SmallInt&, const SmallInt&);
    friend SmallInt operator - (const SmallInt&, const SmallInt&);
    friend SmallInt operator * (const SmallInt&, const SmallInt&);
    friend SmallInt operator / (const SmallInt&, const SmallInt&);
    friend SmallInt operator % (const SmallInt&, const SmallInt&);

    friend SmallInt operator ^ (const SmallInt&, const SmallInt&);
    friend SmallInt operator ^ (const SmallInt&, int);

    friend SmallInt operator - (const SmallInt&); // negation

    friend bool operator == (const SmallInt&, const SmallInt&);
    friend bool operator != (const SmallInt&, const SmallInt&);
    friend bool operator <  (const SmallInt&, const SmallInt&);
    friend bool operator <= (const SmallInt&, const SmallInt&);
    friend bool operator >  (const SmallInt&, const SmallInt&);
    friend bool operator >= (const SmallInt&, const SmallInt&);

    friend bool is_zero(const SmallInt&);
    friend int sign(const SmallInt&);
}; 

inline SmallInt::SmallInt(int a = 0) {
  type = SMALLINT;
  frep = (void *) (new int);
  *((int*)frep) = a;
}

inline int SmallInt::i_value() const {
  return(*((int*)frep));
}

inline int i_value(const SmallInt& si) {
  return(si.i_value());
}

inline void SmallInt::print() { 
  cout << this.i_value() ;
}

inline const char* SmallInt::class_name() {
  return(Class_name); 
}

#endif
