/*************************************************************************

  APU R 2.0

  intpoint.h

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/

#if !defined(INTPOINT_HDR)
#define INTPOINT_HDR

#include "globals.h"
#include "array.h"
extern "C" long lrand48(); 

#define MAX_COOR 100000

/* An n-dimensional point is defined as an n-dimensional array. Usual
array operations are supported. See array.h for array operations.  */ 

typedef Array<int> IntPoint; 

inline bool same_point(IntPoint p1, IntPoint p2) { 
  int n1 = size_of(p1); 
  int n2 = size_of(p2); 
  if (n1 != n2) 
    return false; 
  for (int i=0;i<n1;i++) 
    if (p1[i] != p2[i])
      return false; 
  return true; 
}

inline int dot_product(IntPoint p1,IntPoint p2, int dim) { 
  if ((size_of(p1) < dim) || (size_of(p2) < dim)) { 
    cout << "Error in dot_product(IntPoint,IntPoint) - dimension mismatch\n"; 
    return(0); 
  } 
  int result = 0; 
  for (int i=0;i<dim;i++) 
    result += p1[i] * p2[i]; 
  return result; 
} 

inline IntPoint random_point(int dim) { 
   IntPoint result = Array<int>(dim); 
   for (int i=0;i<dim;i++) 
     result[i] = (int) ((unsigned)(lrand48() % MAX_COOR));  
   return result; 
}

#endif
