
/*************************************************************************

  APU R 2.0

  debug.c

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  
#include "polynomial.h"
#include "slp.h"

Polynomial slp_to_poly(Slp slp); 
Polynomial slppoly_to_poly(const Polynomial&); 

#if DEBUG

Polynomial
slp_to_poly(Slp slp) 
{
  Polynomial result,*res_ptr;
  if (is_zero(slp)) { 
    result = Polynomial::zero_poly;
    return result; 
  }
  if (debug_info_of(slp) == 0) { 
    ((Slp_node*)(slp.frep))->debug_info = new DebugClass; 
  }
  if (debug_info_of(slp)->poly_evaluated == DebugClass::poly_eval_level) 
    result = *(debug_info_of(slp)->poly);
  else if (is_variable(slp)) {
    res_ptr = new Polynomial(variable_of(slp));
    debug_info_of(slp)->poly = res_ptr;
    result = *res_ptr;
    debug_info_of(slp)->poly_evaluated = DebugClass::poly_eval_level;
  }
  else if (is_numerical(slp)) {
    res_ptr = new Polynomial(fe_of(slp));
    debug_info_of(slp)->poly = res_ptr;
    result = *res_ptr;
    debug_info_of(slp)->poly_evaluated = DebugClass::poly_eval_level;
  }
  else {
    Polynomial left = slp_to_poly(lchild(slp));
    Polynomial right = slp_to_poly(rchild(slp));
    switch(op(slp)) {
      case ADD : result = left + right; break;
      case SUB : result = left - right; break;
      case MULT : result = left * right; break;
      case DIV : result = left / right; break;
      case EXP : result = left ^ (((SmallInt*)(&fe_of(rchild(slp))))->i_value()); break;
      default : cout << "Error in slp_to_poly - unknown operator\n";
    }
    res_ptr = new Polynomial;
    *res_ptr = result;
    debug_info_of(slp)->poly = res_ptr;
    debug_info_of(slp)->poly_evaluated = DebugClass::poly_eval_level;
  }
  return(result);
}

Polynomial
slppoly_to_poly(const Polynomial& poly) 
{
  if (is_zeropoly(poly))
    return(poly);
  else if (is_terminal(poly)) {
    if (type_of(fe_of(poly)) == SLP) {
      Polynomial result = slp_to_poly(*((Slp*)(fePtr_of(poly))));
      return(result);
    }
    else return(poly);
  }
  else {
    TermList tl = termlist_of(poly);
    TermNode *lead,*curr;
    lead = 0; curr = 0;
    for (TermNode* temp=leadnodePtr_of(tl);temp!=0;temp=temp->next_of()) {
      if (lead == 0) {
	if (!is_zeropoly(temp->coeff_poly())) {
	  lead = new TermNode(temp->degree_of(),
			      slppoly_to_poly(temp->coeff_poly()));
	  curr = lead;
	}
      }
      else { 
	if (!is_zeropoly(temp->coeff_poly())) {
	  curr->set_next(TermNode(temp->degree_of(),
			      slppoly_to_poly(temp->coeff_poly())));
	  curr = curr->next_of();
	}
      }
    }
    Polynomial result = Polynomial(variable_of(poly),TermList(lead));
    return(result);
  }
}

#endif
