
/*************************************************************************

  APU R 2.0

  init.c

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  
#include "slp.h"
#include "variable.h"
#include "intmodp.h"
#include "polynomial.h"
#include "primelist.h"
#include "multi_intmodp.h"
#include <iostream.h>
#include <fstream.h>
#define NUM_POLYVARS 30
#define NUM_SLPVARS 20

char* poly_vars[NUM_POLYVARS] = {"x1","x2","x3","x4","x5","x6","x7","x8",
"x9","x10","s1","s2","s3","s4","s5","s6","s7","s8","s9","s10","u1","u2",
"u3","u4","u5","u6","u7","u8","u9","u10"}; 

char* slp_vars[NUM_POLYVARS] = {"u1","u2","u3","u4","u5","u6","u7","u8",
"u9","u10","l1","l2","l3","l4","l5","l6","l7","l8","l9","l10"}; 

PrimeList Primes; 
List<Variable> Polynomial::var_list;
List<Variable> Slp::var_list;
const Polynomial Polynomial::zero_poly; 
const TermList TermList::zero_termlist;
const Slp Slp::zero_slp = Slp(0);
Slp Slp::unit_slp = Slp(1); 
int DebugClass::poly_eval_level = 1;
int Polynomial::Num_vars = 0; 
int Slp::Num_vars = 0; 


void apu_init(); 
void init_var();
void init_slp_var();
void init_poly_var();
void init_primes(); 


void apu_init() { 
  // init_var(); 
  init_primes(); 
}

void 
init_var() {
  init_poly_var();
  init_slp_var();
  // exactinit();
}

void
init_slp_var() {

  int i;
  for (i=0;i<NUM_SLPVARS;i++) {
    Variable* temp = new Variable(slp_vars[i]);
    temp->index(i);
    temp->slp(Slp(temp));
    // temp->value(Field_element((int)(random()))); // SmallInt type;
    temp->value(Field_element((int)(i*10+3))); 
    (Slp::var_list).add(*temp);
  }
}

void
init_poly_var() {

  int i;
  for (i=0;i<NUM_POLYVARS;i++) {
    Variable* temp = new Variable(poly_vars[i]);
    temp->index(i);
    temp->slp(Slp(temp));
    // temp->value(Field_element((int)(random()))); // SmallInt type;
    temp->value(Field_element((int)(i*11+3))); 
    (Polynomial::var_list).add(*temp);
  }
}

void
init_primes() { 
  ifstream file;

  file.open("primelist",ios::in); 

  int num_primes; 
  if (!file) { 
    cout << "Cannot open file: primelist\n"; 
    exit(1); 
  }
  file >> num_primes; 

  Primes = PrimeList(num_primes); 
  for (int i=0;i<num_primes;i++) 
    file >> Primes.primes[i]; 
  file.close(); 
  for (int i=0;i<MULTI_NUMPRIMES;i++) 
    MultiIntModp::primes[i] = Primes.primes[i]; 
}
