/* LU decomposition */
/* re-written by I.Z.Emiris (c), based on Numeric Recipes in C	*/
/* 7/1/93: modified: d contains 0.0 iff input was singular	*/
/* 8/24/93: generalized to mxn matrices, m>=n; singular iff rank <n	*/
/* notice indx[] doesnt express LU, but is correct for lubksb()!	*/

#include <stdio.h>
#include <math.h>
#include "nrutild.h"

#define TINY 1.0e-11			/* decreased */

void ludecomp(double** mat,int m,int n,int* indx,double* d)
{
  int i,imax,j,k;
  double big,dum,sum,temp;
  double *vv;
  
  vv=vector(1,m);			/*m*/
  *d=1.0;
  for (i=1;i<=m;i++) {		/*m*/
   	big=0.0;
	for (j=1;j<=n;j++)
		if ((temp=fabs(mat[i][j])) > big) big=temp;
	if (big == 0.0) {		/* singular matrix */
	  *d = 0.0;
	  return;
	}
	vv[i]=1.0/big;
  }
  for (j=1;j<=n;j++) {
	for (i=1;i<j;i++) {
		sum=mat[i][j];
		for (k=1;k<i;k++) sum -= mat[i][k]*mat[k][j];
		mat[i][j]=sum;
	}
	big=0.0;
	for (i=j;i<=m;i++) {		/*m*/
		sum=mat[i][j];
		for (k=1;k<j;k++) sum -= mat[i][k]*mat[k][j];
		mat[i][j]=sum;
		if ( (dum=vv[i]*fabs(sum)) >= big) {
			big=dum;
			imax=i;
		}
	}
	if (j != imax) {		/* imax >= j */
		for (k=1;k<=n;k++) {
			dum=mat[imax][k];
			mat[imax][k]=mat[j][k];
			mat[j][k]=dum;
		}
		*d = -(*d);
		vv[imax]=vv[j];
	}
	indx[j]=imax;
	if ( fabs(mat[j][j]) < TINY) {	/* changed here */
		*d = 0.0;
		return;
	}
    /* singular matrix */
    if (j != n) {
      dum=1.0/(mat[j][j]);
      for (i=j+1;i<=m;i++) mat[i][j] *= dum;
    }
  }
  free_vector(vv,1,m);
  for( i=1; i<=n; i++) {			/* extra singularity test */
    if( fabs(mat[i][i]) < TINY) {
      *d = 0.0;
      fprintf(stderr,"\nWARN ludecomp! didnt catch nonsingularity at %d\n\n",i);
      fflush(stderr);
      return;
    }
  }/*i*/
}/*ludecomp*/

#undef TINY
