
/*************************************************************************

  APU R 2.0

  det_sign.c

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  
#include "poly_sign.h"
#include "det_sign.h"
extern int Debug;

/* Returns the product of p' and the q poly's raised to corresponding
   exponents given in exp_array */
Polynomial
compute_product (const Polynomial& p, const List<Polynomial>& qlist, 
		 Array<int> exp_array)
{

  Polynomial result = diff(p);
  ListNode<Polynomial>* temp = head_of(qlist);
  for (int i=0;i<size_of(exp_array);i++) {
    result = result*((temp->item_of())^exp_array[i]);
    temp = temp->next_of();
  }
  return(result);
}

/* The s_hat array contains exponents. These are used to determine
   the product of the q polynomials which needs to be computed. 
   That is, the q product is : qprod = q0^(s_hat[0]).q1^(s_hat[1]) ...
   The q product is multiplied by the derivative of p to get
   p'.qprod.
   The routine returns the difference in alternations of the
   Sturm-Habicht sequence of p and p'.qprod. 
*/

Array<int>
sturm_query_values (const Polynomial& p, const List<Polynomial>& qlist, 
		    Matrix<int> s_hat)
{
  int dim = dim1_of(s_hat);
  Array<int> result = Array<int>(dim);
  for (int i=0;i<dim;i++) {
    Polynomial polyprod = compute_product(p,qlist,s_hat[i]);
    List<Polynomial> sturmseq = sturm_habicht(p,polyprod);
    result[i] = diff_in_alternations(sturmseq);
  }
  return(result);
}

Matrix<int>
det_sign (const Polynomial& p, List<Polynomial>& qlist)
{
   Matrix<int> H = get_H1();
   Matrix<int> r_hat = r_hat_startup();
   Matrix<int> s_hat = s_hat_startup();
   Array<int> s = sturm_query_values(p,qlist,s_hat);
   Array<int> r = lin_solve(H,s);
   r_hat = eliminate_r_hat_zero_entries(r_hat,r);
   r = eliminate_zero_entries(r);

   ListNode<Polynomial>* temp = head_of(qlist); 
   for  (temp = temp->next_of(); temp != 0; temp = temp->next_of()) {
     if (is_empty(r_hat)) {
       H = get_H1();
       r_hat = r_hat_startup();
       s_hat = s_hat_startup();
     }
     else {
       r_hat = augment_r_hat(r_hat);
       Matrix<int> rowlist = compute_rows(r_hat);
       H = compute_Hmatrix(rowlist,r_hat);
       s_hat = update_sturm_queries(rowlist);
     }
     s = sturm_query_values(p,qlist,s_hat);
     r = lin_solve(H,s);
     r_hat = eliminate_r_hat_zero_entries(r_hat,r);
     r = eliminate_zero_entries(r);
   }
   return(r_hat);
}

Matrix<int>
get_H1()
{

  Matrix<int> result = Matrix<int>(3,3);
  result[0][0] = 1; result[0][1] =  1; result[0][2] = 1;
  result[1][0] = 1; result[1][1] = -1; result[1][2] = 0;
  result[2][0] = 1; result[2][1] =  1; result[2][2] = 0;
  return(result);
}

