
/*************************************************************************

  APU R 2.0

  poly_interpmem.c

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  
#include "poly_interp.h"
#include "array.h"

extern int Debug;
extern Slp poly_to_slp(Polynomial); 

Polynomial
vanderpolate_mem(Array<Polynomial> vals_array, Array<Field_element> eval_points,
	     int n, char* var, int poly_type = INTMODP) 
{
  Array<Polynomial> master_coeffs = 
	  get_masterpoly_coeffs(n,poly_type,eval_points); 
  Polynomial master_poly = get_master_poly(master_coeffs,n,var);
  if (Debug) 
    cout << master_poly << "\n"; 
  Polynomial Qi_sym = get_sym_quotient_poly(var,n,poly_type); 
  Polynomial divisor_i, Qi, Pi; 
  Field_element Qi_eval;
  Array<Field_element> coeff_array = Array<Field_element>(n+1); 
  for (int i=0;i<=n;i++) {
    divisor_i = get_ith_divisor(var,eval_points[i],poly_type);
    get_ith_quotient(&Qi_sym,divisor_i,master_poly,i);
    if (Debug) 
      cout << Qi_sym << "\n"; 
    Qi_eval = get_Qi_eval(i, eval_points, n, poly_type);  
    if (Debug) 
      cout << Qi_eval << "\n"; 
    // Pi = Qi/Qi_eval;
    update_coeffs(coeff_array,Qi_sym,Qi_eval,vals_array,i,n,var,poly_type); 
    if (Debug) 
      cout << coeff_array;

  }
  Polynomial result = make_interpolated_poly(coeff_array,n,var,poly_type); 
  return result;
}

Polynomial
vanderpolate_mem2(Array<Polynomial> vals_array, Array<Field_element> 
eval_points, int n, char* var, Polynomial* mp_ptr, Polynomial* Qi_sym_ptr, 
Array<Field_element> coeff_array,int poly_type = INTMODP) 
{
  Polynomial master_poly = *mp_ptr; 
  if (Debug) 
    cout << master_poly << "\n"; 
  Polynomial Qi_sym = *Qi_sym_ptr; 
  Polynomial divisor_i, Qi, Pi; 
  Field_element Qi_eval;
  for (int i=0;i<=n;i++) {
    divisor_i = get_ith_divisor(var,eval_points[i],poly_type);
    if (Debug) 
      cout << divisor_i << "\n"; 
    get_ith_quotient(Qi_sym_ptr,divisor_i,master_poly,i);
    if (Debug) 
      cout << Qi_sym << "\n"; 
    Qi_eval = get_Qi_eval(i, eval_points, n, poly_type);  
    if (Debug) 
      cout << Qi_eval << "\n"; 
    // Pi = Qi/Qi_eval;
    update_coeffs(coeff_array,Qi_sym,Qi_eval,vals_array,i,n,var,poly_type); 
    if (Debug) 
      cout << coeff_array;

  }
  Polynomial result = make_interpolated_poly(coeff_array,n,var,poly_type); 
  return result;
}

/* Returns a placeholder polynomial of degree n. The memory allocated here
   is reused in each iteration. */
Polynomial 
get_sym_quotient_poly(char* var, int n, int poly_type) 
{
  Field_element temp_fe = make_Field_element(1,poly_type); 
  Polynomial coeff_poly = Polynomial(temp_fe); 
  TermNode* tn; TermNode* curr; 

  tn = new TermNode(n, coeff_poly); 
  curr = tn; 

  for (int i=n-1;i>=0;i--) { 
    temp_fe = make_Field_element(1,poly_type); 
    coeff_poly = Polynomial(temp_fe); 
    curr->set_next(TermNode(i,coeff_poly)); 
    curr = curr->next_of(); 
  }
  Polynomial result = Polynomial(var,TermList(tn)); 
  return result; 
}

void
get_ith_quotient(Polynomial* qi_ptr,Polynomial divisor, 
		 Polynomial master_poly,int i) 
{
  Field_element temp; 
  TermNode *tn_mp, *tn_div, *tn_qi; 


  tn_mp = termlist_of(master_poly).tnode_of();
  tn_div = (termlist_of(divisor).tnode_of())->next_of(); 
  tn_qi = termlist_of(*qi_ptr).tnode_of(); 

  Field_element curr_lc = (tn_mp->coeff_poly()).fe_of();

  *((tn_qi->coeff_poly()).fePtr_of()) = curr_lc; 

  for (tn_qi=tn_qi->next_of();tn_qi!=0; tn_qi=tn_qi->next_of()) { 
    temp = (tn_div->coeff_poly()).fe_of() * curr_lc; 
    tn_mp = tn_mp->next_of();
    curr_lc = (tn_mp->coeff_poly()).fe_of() - temp; 
    *((tn_qi->coeff_poly()).fePtr_of()) = curr_lc; 
    if (Debug) 
      cout << *qi_ptr << "\n";

  }

}

/* Returns the coefficients of the "master" polynomial, which is 
   defined as (z-e_0)*...*(z-e_n) where e_i is the ith point _at_
   which the polynomial has been evaluated. */ 
/* The array "result" is assumed to have been already allocated. 
   This routine is used for better memory management as part of
   the "mem" version of vanderpolate() */
void
get_mp_coeffs_mem(Array<Field_element> result, int n,int poly_type, 
Array<Field_element> eval_points)
{
  if (Debug) { 
    for (int i=0;i<size_of(eval_points);i++) 
      cout << eval_points[i] << "\n";
  }
  result[0] = make_Field_element(1,poly_type); 
  result[1] = - eval_points[0]; 
  for (int i=2;i<n+2;i++) 
    result[i] = make_Field_element(0,poly_type); 
  Field_element temp, temp2; 
  for (int i=1;i<n+1;i++) {
    temp2 = result[0];
    for (int j=1;j<=i+1;j++) {
      temp = - eval_points[i]*temp2 + result[j];
      temp2 = result[j];
      result[j] = temp;
      if (Debug) 
	cout << result[j] << "\n";
    }
  }
  if (Debug) { 
    for (int i=0;i<size_of(result);i++) 
      cout << result[i] << "\n"; 
  }
}

/* The array "result" is assumed to have been already allocated. 
   This routine is used for better memory management as part of
   the "mem" version of vanderpolate() */
void
get_masterpoly_coeffs_mem(Array<Polynomial> result,int n,
	   Array<Field_element> mp_coeffs) 
{
  for (int i=0;i<n+2;i++) 
    result[i] = Polynomial(mp_coeffs[i]);
}

/* The polynomial "result" is assumed to have been already allocated. 
   This routine is used for better memory management as part of
   the "mem" version of vanderpolate() */
void
get_master_poly_mem(Polynomial* result,Array<Polynomial> master_coeffs, 
		int n, char* var) 
{
  Polynomial mpoly = *result; 
  TermList tl = termlist_of(mpoly);  
  if (tl.tnode_of() == 0) { 
    tl = TermNode(n+1,master_coeffs[0]); 
    TermNode* tnodeP = tl.last(); 
    *result = Polynomial(var,tl);  
    for (int i=1;i<n+2;i++) {
      tnodeP->set_next(TermNode(n+1-i,master_coeffs[i])); 
      tnodeP = tnodeP->next_of();
    }
  }
  else { 
    TermNode* tnodeP = tl.tnode_of(); 
    for (int i=0;i<n+2;i++) { 
      *((tnodeP->coeff_poly()).fePtr_of()) = *(master_coeffs[i].fePtr_of());
      tnodeP = tnodeP->next_of(); 
    }
  }
  if (Debug) { 
    cout << *result << "\n"; 
  }
}

