
/*************************************************************************

  APU R 2.0

  termlist.c

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  
#include "polynomial.h"

extern int Debug;

TermList
operator + (const TermList& t1, const TermList& t2) {
  TermNode *temp1, *temp2; 
  TermNode *lead, *curr;
  Polynomial temp_poly;

  temp1 = t1.tnode; 
  temp2 = t2.tnode; 

  lead = 0;
  while ((lead == 0) && (temp1 != 0) && (temp2 != 0)) {
    if (temp1->degree_of() > temp2->degree_of()) {
      if (!is_zeropoly(temp1->coeff_poly()))
        lead = new TermNode(temp1->degree_of(),temp1->coeff_poly()); 
      temp1 = temp1->next_of();
    }
    else if (temp1->degree_of() < temp2->degree_of()) {
      if (!is_zeropoly(temp2->coeff_poly()))
        lead = new TermNode(temp2->degree_of(),temp2->coeff_poly()); 
      temp2 = temp2->next_of();
    }
    else { 
      temp_poly = temp1->coeff_poly() + temp2->coeff_poly();
      if (!is_zeropoly(temp_poly))
        lead = new TermNode(temp2->degree_of(),temp_poly);
      temp1 = temp1->next_of();
      temp2 = temp2->next_of();
    }
  }
  if (lead == 0) {
    if (temp1 == 0) {
      while ((lead == 0) && (temp2 != 0)) {
	if (!is_zeropoly(temp2->coeff_poly()))
	  lead = new TermNode(temp2->degree_of(),temp2->coeff_poly());
	temp2 = temp2->next_of();
      }
    }
    else if (temp2 == 0) {
      while ((lead == 0) && (temp1 != 0)) {
	if (!is_zeropoly(temp1->coeff_poly()))
	  lead = new TermNode(temp1->degree_of(),temp1->coeff_poly());
	temp1 = temp1->next_of();
      }
    }
    else cout << "Error in + (TermList, TermList)\n";
  }
  if (lead == 0) 
    return(TermList::zero_termlist);
  else curr = lead;

  while ( (temp1!=0) && (temp2!=0) ) {
    if (temp1->degree_of() > temp2->degree_of()) {
      if (!is_zeropoly(temp1->coeff_poly())) {
        curr->set_next(TermNode(temp1->degree_of(),temp1->coeff_poly())); 
        curr = curr->next_of();
      }
      temp1 = temp1->next_of(); 
    }
    else if (temp1->degree_of() < temp2->degree_of()) {
      if (!is_zeropoly(temp2->coeff_poly())) {
        curr->set_next(TermNode(temp2->degree_of(),temp2->coeff_poly())); 
        curr = curr->next_of();
      }
      temp2 = temp2->next_of(); 
    }
    else {
      temp_poly = temp1->coeff_poly() + temp2->coeff_poly();
      if (!is_zeropoly(temp_poly)) {
        curr->set_next(TermNode(temp1->degree_of(),temp_poly));
        curr = curr->next_of();
      }
      temp1 = temp1->next_of(); 
      temp2 = temp2->next_of(); 
    }
  }
  while (temp1 != 0) { 
    if (!is_zeropoly(temp1->coeff_poly())) {
      curr->set_next(TermNode(temp1->degree_of(),temp1->coeff_poly())); 
      curr = curr->next_of();
    }
    temp1 = temp1->next_of(); 
  }
  while (temp2 != 0) {
    if (!is_zeropoly(temp2->coeff_poly())) {
      curr->set_next(TermNode(temp2->degree_of(),temp2->coeff_poly())); 
      curr = curr->next_of();
    }
    temp2 = temp2->next_of(); 
  }
  TermList result = TermList(lead);;
  return(result);
}

TermList
operator + (const TermList& tl, const Field_element& fe) {
  TermNode *temp, *lead, *curr;
  Polynomial temp_poly;

  temp = tl.tnode; 
  lead = 0;
  while ((lead == 0) && (temp != 0)) {
    if (temp->degree_of() > 0) {
      if (!is_zeropoly(temp->coeff_poly()))
        lead = new TermNode(temp->degree_of(),temp->coeff_poly()); 
      temp = temp->next_of();
    }
    else { 
      temp_poly = temp->coeff_poly() + fe; 
      if (!is_zeropoly(temp_poly))
        lead = new TermNode(temp->degree_of(),temp_poly);
      temp = temp->next_of();
    }
  }
  if (lead == 0) {
    if (temp == 0) {
	if (!is_zero(fe))
	  lead = new TermNode(0,Polynomial(fe));
    }
    else cout << "Error in + (TermList, Field_element)\n";
  }
  if (lead == 0) 
    return(TermList::zero_termlist);
  else curr = lead;

  while (temp != 0)  {
    if (temp->degree_of() > 0) {
      if (!is_zeropoly(temp->coeff_poly())) {
        curr->set_next(TermNode(temp->degree_of(),temp->coeff_poly())); 
        curr = curr->next_of();
      }
      temp = temp->next_of(); 
    }
    else {
      temp_poly = temp->coeff_poly() + fe; 
      if (!is_zeropoly(temp_poly)) {
        curr->set_next(TermNode(temp->degree_of(),temp_poly));
        curr = curr->next_of();
      }
      temp = temp->next_of(); 
    }
  }
  if (curr->degree_of() > 0) 
    curr->set_next(TermNode(0,Polynomial(fe)));

  TermList result = TermList(lead);  
  return(result);
}

TermList
operator + (const TermList& tl, const Polynomial& poly) {
  TermNode *temp, *lead, *curr;
  Polynomial temp_poly;

  temp = tl.tnode; 
  lead = 0;
  while ((lead == 0) && (temp != 0)) {
    if (temp->degree_of() > 0) {
      if (!is_zeropoly(temp->coeff_poly()))
        lead = new TermNode(temp->degree_of(),temp->coeff_poly()); 
      temp = temp->next_of();
    }
    else { 
      temp_poly = temp->coeff_poly() + poly; 
      if (!is_zeropoly(temp_poly))
        lead = new TermNode(temp->degree_of(),temp_poly);
      temp = temp->next_of();
    }
  }
  if (lead == 0) {
    if (temp == 0) {
	if (!is_zeropoly(poly))
	  lead = new TermNode(0,poly);
    }
    else cout << "Error in + (TermList, Field_element)\n";
  }
  if (lead == 0) 
    return(TermList::zero_termlist);
  else curr = lead;

  while (temp != 0)  {
    if (temp->degree_of() > 0) {
      if (!is_zeropoly(temp->coeff_poly())) {
        curr->set_next(TermNode(temp->degree_of(),temp->coeff_poly())); 
        curr = curr->next_of();
      }
      temp = temp->next_of(); 
    }
    else {
      temp_poly = temp->coeff_poly() + poly; 
      if (!is_zeropoly(temp_poly)) {
        curr->set_next(TermNode(temp->degree_of(),temp_poly));
        curr = curr->next_of();
      }
      temp = temp->next_of(); 
    }
  }
  if (curr->degree_of() > 0) 
    curr->set_next(TermNode(0,poly));

  TermList result = TermList(lead);  
  return(result);
}

TermList
operator - (const TermList& t1, const TermList& t2) {
  TermNode *temp1, *temp2; 
  TermNode *lead, *curr;
  Polynomial temp_poly;

  temp1 = t1.tnode; 
  temp2 = t2.tnode; 

  lead = 0;
  while ((lead == 0) && (temp1 != 0) && (temp2 != 0)) {
    if (temp1->degree_of() > temp2->degree_of()) {
      if (!is_zeropoly(temp1->coeff_poly()))
        lead = new TermNode(temp1->degree_of(),temp1->coeff_poly()); 
      temp1 = temp1->next_of();
    }
    else if (temp1->degree_of() < temp2->degree_of()) {
      if (!is_zeropoly(temp2->coeff_poly()))
        lead = new TermNode(temp2->degree_of(),-(temp2->coeff_poly())); 
      temp2 = temp2->next_of();
    }
    else { 
      temp_poly = temp1->coeff_poly() - temp2->coeff_poly();
      if (!is_zeropoly(temp_poly))
        lead = new TermNode(temp2->degree_of(),temp_poly);
      temp1 = temp1->next_of();
      temp2 = temp2->next_of();
    }
  }
  if (lead == 0) {
    if (temp1 == 0) {
      while ((lead == 0) && (temp2 != 0)) {
	if (!is_zeropoly(temp2->coeff_poly()))
	  lead = new TermNode(temp2->degree_of(),-(temp2->coeff_poly()));
	temp2 = temp2->next_of();
      }
    }
    else if (temp2 == 0) {
      while ((lead == 0) && (temp1 != 0)) {
	if (!is_zeropoly(temp1->coeff_poly()))
	  lead = new TermNode(temp1->degree_of(),temp1->coeff_poly());
	temp1 = temp1->next_of();
      }
    }
    else cout << "Error in + (TermList, TermList)\n";
  }
  if (lead == 0) 
    return(TermList::zero_termlist);
  else curr = lead;

  while ( (temp1!=0) && (temp2!=0) ) {
    if (temp1->degree_of() > temp2->degree_of()) {
      if (!is_zeropoly(temp1->coeff_poly())) {
        curr->set_next(TermNode(temp1->degree_of(),temp1->coeff_poly())); 
        curr = curr->next_of();
      }
      temp1 = temp1->next_of(); 
    }
    else if (temp1->degree_of() < temp2->degree_of()) {
      if (!is_zeropoly(temp2->coeff_poly())) {
        curr->set_next(TermNode(temp2->degree_of(),-(temp2->coeff_poly()))); 
        curr = curr->next_of();
      }
      temp2 = temp2->next_of(); 
    }
    else {
      temp_poly = temp1->coeff_poly() - temp2->coeff_poly();
      if (!is_zeropoly(temp_poly)) {
        curr->set_next(TermNode(temp1->degree_of(),temp_poly));
        curr = curr->next_of();
      }
      temp1 = temp1->next_of(); 
      temp2 = temp2->next_of(); 
    }
  }
  while (temp1 != 0) { 
    if (!is_zeropoly(temp1->coeff_poly())) {
      curr->set_next(TermNode(temp1->degree_of(),temp1->coeff_poly())); 
      curr = curr->next_of();
    }
    temp1 = temp1->next_of(); 
  }
  while (temp2 != 0) {
    if (!is_zeropoly(temp2->coeff_poly())) {
      curr->set_next(TermNode(temp2->degree_of(),-(temp2->coeff_poly()))); 
      curr = curr->next_of();
    }
    temp2 = temp2->next_of(); 
  }
  TermList result = TermList(lead);;
  return(result);
}

TermList
operator - (const TermList& tl, const Field_element& fe) {
  TermNode *temp, *lead, *curr;
  Polynomial temp_poly;

  temp = tl.tnode; 
  lead = 0;
  while ((lead == 0) && (temp != 0)) {
    if (temp->degree_of() > 0) {
      if (!is_zeropoly(temp->coeff_poly()))
        lead = new TermNode(temp->degree_of(),temp->coeff_poly()); 
      temp = temp->next_of();
    }
    else { 
      temp_poly = temp->coeff_poly() - fe; 
      if (!is_zeropoly(temp_poly))
        lead = new TermNode(temp->degree_of(),temp_poly);
      temp = temp->next_of();
    }
  }
  if (lead == 0) {
    if (temp == 0) {
	if (!is_zero(fe))
	  lead = new TermNode(0,Polynomial(-fe));
    }
    else cout << "Error in + (TermList, Field_element)\n";
  }
  if (lead == 0) 
    return(TermList::zero_termlist);
  else curr = lead;

  while (temp != 0)  {
    if (temp->degree_of() > 0) {
      if (!is_zeropoly(temp->coeff_poly())) {
        curr->set_next(TermNode(temp->degree_of(),temp->coeff_poly())); 
        curr = curr->next_of();
      }
      temp = temp->next_of(); 
    }
    else {
      temp_poly = temp->coeff_poly() - fe; 
      if (!is_zeropoly(temp_poly)) {
        curr->set_next(TermNode(temp->degree_of(),temp_poly));
        curr = curr->next_of();
      }
      temp = temp->next_of(); 
    }
  }
  if (curr->degree_of() > 0) 
    curr->set_next(TermNode(0,Polynomial(-fe)));

  TermList result = TermList(lead);  
  return(result);
}

TermList
operator - (const Field_element& fe, const TermList& tl) {
  TermNode *temp, *lead, *curr;
  Polynomial temp_poly;

  temp = tl.tnode; 
  lead = 0;
  while ((lead == 0) && (temp != 0)) {
    if (temp->degree_of() > 0) {
      if (!is_zeropoly(temp->coeff_poly()))
        lead = new TermNode(temp->degree_of(),-(temp->coeff_poly())); 
      temp = temp->next_of();
    }
    else { 
      temp_poly = fe - temp->coeff_poly(); 
      if (!is_zeropoly(temp_poly))
        lead = new TermNode(temp->degree_of(),temp_poly);
      temp = temp->next_of();
    }
  }
  if (lead == 0) {
    if (temp == 0) {
	if (!is_zero(fe))
	  lead = new TermNode(0,Polynomial(fe));
    }
    else cout << "Error in + (TermList, Field_element)\n";
  }
  if (lead == 0) 
    return(TermList::zero_termlist);
  else curr = lead;

  while (temp != 0)  {
    if (temp->degree_of() > 0) {
      if (!is_zeropoly(temp->coeff_poly())) {
        curr->set_next(TermNode(temp->degree_of(),-(temp->coeff_poly()))); 
        curr = curr->next_of();
      }
      temp = temp->next_of(); 
    }
    else {
      temp_poly = fe - temp->coeff_poly(); 
      if (!is_zeropoly(temp_poly)) {
        curr->set_next(TermNode(temp->degree_of(),temp_poly));
        curr = curr->next_of();
      }
      temp = temp->next_of(); 
    }
  }
  if (curr->degree_of() > 0) 
    curr->set_next(TermNode(0,Polynomial(fe)));

  TermList result = TermList(lead);  
  return(result);
}

TermList
operator - (const TermList& tl, const Polynomial& poly) {
  TermNode *temp, *lead, *curr;
  Polynomial temp_poly;

  temp = tl.tnode; 
  lead = 0;
  while ((lead == 0) && (temp != 0)) {
    if (temp->degree_of() > 0) {
      if (!is_zeropoly(temp->coeff_poly()))
        lead = new TermNode(temp->degree_of(),temp->coeff_poly()); 
      temp = temp->next_of();
    }
    else { 
      temp_poly = temp->coeff_poly() - poly; 
      if (!is_zeropoly(temp_poly))
        lead = new TermNode(temp->degree_of(),temp_poly);
      temp = temp->next_of();
    }
  }
  if (lead == 0) {
    if (temp == 0) {
	if (!is_zeropoly(poly))
	  lead = new TermNode(0,-poly);
    }
    else cout << "Error in + (TermList, Field_element)\n";
  }
  if (lead == 0) 
    return(TermList::zero_termlist);
  else curr = lead;

  while (temp != 0)  {
    if (temp->degree_of() > 0) {
      if (!is_zeropoly(temp->coeff_poly())) {
        curr->set_next(TermNode(temp->degree_of(),temp->coeff_poly())); 
        curr = curr->next_of();
      }
      temp = temp->next_of(); 
    }
    else {
      temp_poly = temp->coeff_poly() - poly; 
      if (!is_zeropoly(temp_poly)) {
        curr->set_next(TermNode(temp->degree_of(),temp_poly));
        curr = curr->next_of();
      }
      temp = temp->next_of(); 
    }
  }
  if (curr->degree_of() > 0) 
    curr->set_next(TermNode(0,-poly));

  TermList result = TermList(lead);  
  return(result);
}

TermList
operator - (const Polynomial& poly, const TermList& tl) {
  TermNode *temp, *lead, *curr;
  Polynomial temp_poly;

  temp = tl.tnode; 
  lead = 0;
  while ((lead == 0) && (temp != 0)) {
    if (temp->degree_of() > 0) {
      if (!is_zeropoly(temp->coeff_poly()))
        lead = new TermNode(temp->degree_of(),-(temp->coeff_poly())); 
      temp = temp->next_of();
    }
    else { 
      temp_poly = poly - temp->coeff_poly(); 
      if (!is_zeropoly(temp_poly))
        lead = new TermNode(temp->degree_of(),temp_poly);
      temp = temp->next_of();
    }
  }
  if (lead == 0) {
    if (temp == 0) {
	if (!is_zeropoly(poly))
	  lead = new TermNode(0,poly);
    }
    else cout << "Error in + (TermList, Field_element)\n";
  }
  if (lead == 0) 
    return(TermList::zero_termlist);
  else curr = lead;

  while (temp != 0)  {
    if (temp->degree_of() > 0) {
      if (!is_zeropoly(temp->coeff_poly())) {
        curr->set_next(TermNode(temp->degree_of(),-(temp->coeff_poly()))); 
        curr = curr->next_of();
      }
      temp = temp->next_of(); 
    }
    else {
      temp_poly = poly - temp->coeff_poly(); 
      if (!is_zeropoly(temp_poly)) {
        curr->set_next(TermNode(temp->degree_of(),temp_poly));
        curr = curr->next_of();
      }
      temp = temp->next_of(); 
    }
  }
  if (curr->degree_of() > 0) 
    curr->set_next(TermNode(0,poly));

  TermList result = TermList(lead);  
  return(result);
}

TermList
operator * (const TermList& t1, const TermList& t2) {
  TermList result, temp;
  TermNode *temp1, *temp2; 
  TermNode *lead, *curr;
  Polynomial temp_poly;

  lead = 0;
  temp1 = t1.tnode; 
  while (temp1 != 0) {
    temp2 = t2.tnode;
    while (temp2 != 0) {
      temp_poly = temp1->coeff_poly() * temp2->coeff_poly();
      if (!is_zeropoly(temp_poly)) {
	    if (lead == 0) {
              lead = new TermNode(temp1->degree_of() + temp2->degree_of(),
				  temp_poly);
	      curr = lead; 
	    }
	    else {
	      curr->set_next(TermNode(temp1->degree_of() + temp2->degree_of(),
				      temp_poly));
	      curr = curr->next_of();
	    }
      }
      temp2 = temp2->next_of();
    }
    temp1 = temp1->next_of();
    if (lead == 0) temp = TermList::zero_termlist;
    else temp = TermList(lead);
    if (is_zerotermlist(result))
      result = temp;
    else result = result + temp;
    lead = 0;
  }
  return(result);
}

TermList
operator * (const TermList& tl, const Field_element& fe) {
  TermNode *temp, *lead, *curr;
  Polynomial temp_poly;

  if (is_zero(fe))
    return(TermList::zero_termlist);
  lead = 0;
  temp = tl.tnode; 
  while (temp != 0) {
      temp_poly = temp->coeff_poly() * fe;
      if (!is_zeropoly(temp_poly)) {
	    if (lead == 0) {
              lead = new TermNode(temp->degree_of(),temp_poly);
	      curr = lead; 
	    }
	    else {
	      curr->set_next(TermNode(temp->degree_of(),temp_poly));
	      curr = curr->next_of();
	    }
      }
      temp = temp->next_of();
  }
  if (Debug) cout << TermList(lead) << "\n";
  if (lead == 0) 
    return(TermList::zero_termlist);
  else 
    return(TermList(lead));
}

TermList
operator * (const TermList& tl, const Polynomial& poly) {
  TermNode *temp, *lead, *curr;
  Polynomial temp_poly;

  lead = 0;
  temp = tl.tnode; 
  if (is_zeropoly(poly))
    return(TermList::zero_termlist);
  while (temp != 0) {
      temp_poly = temp->coeff_poly() * poly;
      if (!is_zeropoly(temp_poly)) {
	    if (lead == 0) {
              lead = new TermNode(temp->degree_of(),temp_poly);
	      curr = lead; 
	    }
	    else {
	      curr->set_next(TermNode(temp->degree_of(),temp_poly));
	      curr = curr->next_of();
	    }
      }
      temp = temp->next_of();
  }
  if (Debug) cout << TermList(lead) << "\n";
  if (lead == 0) return(TermList::zero_termlist);
  else return(TermList(lead));
}

TermList 
operator / (const TermList& t1, const TermList& t2) {
  int n = t1.lead_degree();
  int m = t2.lead_degree();

  if (n<m) 
    return(TermList::zero_termlist);
 
  TermList quotient,cur_dividend,temp; 
  TermNode *lead, *curr, *temp1;
  Polynomial temp_poly;

  temp1 = t1.tnode; 
  int done = 0;
  lead = 0;
  while (!done) {
    if (!is_zeropoly(temp1->coeff_poly())) {
      n = temp1->degree_of();
      if (n < m) 
	return(TermList::zero_termlist);
      else { 
	temp_poly = (temp1->coeff_poly())/(t2.lead_coeff());
	if (!is_zeropoly(temp_poly))
	  lead = new TermNode(n-m,temp_poly);
      }
      done = 1;
    }
    else {
      temp1 = temp1->next_of();
      if (temp1 == 0) done = 1;
    }
  }
  if (lead != 0) {
    quotient = TermList(lead);
    temp = quotient * t2; 
    cur_dividend = t1 - temp;
    temp = cur_dividend/t2;
    quotient = quotient + temp;
    return(quotient);
  }
  else return(TermList::zero_termlist);
}

TermList operator / (const Field_element& fe, const TermList& tl) {

  if (tl.lead_degree() > 0) 
    return(TermList::zero_termlist);
  else if (is_zero(fe)) 
    return(TermList::zero_termlist);
  else {
    Polynomial temp_poly = fe/tl.lead_coeff();
    if (!is_zeropoly(temp_poly)) {
      TermNode* lead = new TermNode(0,temp_poly);
      return(TermList(lead));
    }
    else return(TermList::zero_termlist);
  }
}

TermList operator / (const Polynomial& poly, const TermList& tl) {

  if (tl.lead_degree() > 0) 
    return(TermList::zero_termlist);
  else if (is_zeropoly(poly)) 
    return(TermList::zero_termlist);
  else {
    Polynomial temp_poly = poly/tl.lead_coeff();
    if (!is_zeropoly(temp_poly)) {
      TermNode* lead = new TermNode(0,temp_poly);
      return(TermList(lead));
    }
    else return(TermList::zero_termlist);
  }
}

TermList operator / (const TermList& tl, const Polynomial& poly) {
  
  TermNode *temp, *lead, *curr;

  if (is_zeropoly(poly)) {
    cout << "Error in / (TermList,Polynomial) - divide by zero\n";
    return(TermList::zero_termlist);
  }
  Polynomial temp_poly;
  temp = tl.tnode;
  lead = 0;
  while (temp != 0) {
    temp_poly = temp->coeff_poly()/poly;
    if (!is_zeropoly(temp_poly)) {
      if (lead == 0) {
	lead = new TermNode(temp->degree_of(),temp_poly);
	curr = lead; 
      }
      else {
	curr->set_next(TermNode(temp->degree_of(),temp_poly));
	curr = curr->next_of();
      }
    }
    temp = temp->next_of();
  }
  if (lead == 0) return(TermList::zero_termlist);
  else return(TermList(lead));
}

TermList operator / (const TermList& tl, const Field_element& fe) {
  
  TermNode *temp, *lead, *curr;

  if (is_zero(fe)) {
    cout << "Error in / (TermList,Field_element) - divide by zero\n";
    return(TermList::zero_termlist);
  }
  Polynomial temp_poly;
  temp = tl.tnode;
  lead = 0;
  while (temp != 0) {
    temp_poly = temp->coeff_poly()/fe;
    if (!is_zeropoly(temp_poly)) {
      if (lead == 0) {
	lead = new TermNode(temp->degree_of(),temp_poly);
	curr = lead; 
      }
      else {
	curr->set_next(TermNode(temp->degree_of(),temp_poly));
	curr = curr->next_of();
      }
    }
    temp = temp->next_of();
  }
  if (lead == 0) return(TermList::zero_termlist);
  else return(TermList(lead));
}

TermList 
operator % (const TermList& t1, const TermList& t2) {
  int n = t1.lead_degree();
  int m = t2.lead_degree();

  if (n<m) 
    return(t1);
 
  TermList quotient,cur_dividend,temp; 
  TermNode *lead, *curr, *temp1;
  Polynomial temp_poly;

  temp1 = t1.tnode; 
  int done = 0;
  lead = 0;
  while (!done) {
    if (!is_zeropoly(temp1->coeff_poly())) {
      n = temp1->degree_of();
      if (n < m) 
	return(TermList(temp1));
      else { 
	temp_poly = (temp1->coeff_poly())/(t2.lead_coeff());
	if (!is_zeropoly(temp_poly))
	  lead = new TermNode(n-m,temp_poly);
      }
      done = 1;
    }
    else {
      temp1 = temp1->next_of();
      if (temp1 == 0) done = 1;
    }
  }
  if (lead != 0) {
    quotient = TermList(lead);
    temp = quotient * t2; 
    cur_dividend = t1 - temp;
    temp = cur_dividend % t2;
    return(temp);
  }
  else return(t1);
}

TermList operator % (const Field_element& fe, const TermList& tl) {

  if (tl.lead_degree() > 0) { 
    TermNode* lead = new TermNode(0,Polynomial(fe)); 
    return(TermList(lead));
  }
  else if (is_zero(fe)) 
    return(TermList::zero_termlist);
  else {
    Polynomial temp_poly = fe % tl.lead_coeff();
    if (!is_zeropoly(temp_poly)) {
      TermNode* lead = new TermNode(0,temp_poly);
      return(TermList(lead));
    }
    else return(TermList::zero_termlist);
  }
}

TermList operator % (const Polynomial& poly, const TermList& tl) {

  if (tl.lead_degree() > 0) {
    TermNode* lead = new TermNode(0,poly); 
    return(TermList(lead));
  }
  else if (is_zeropoly(poly)) 
    return(TermList::zero_termlist);
  else {
    Polynomial temp_poly = poly % tl.lead_coeff();
    if (!is_zeropoly(temp_poly)) {
      TermNode* lead = new TermNode(0,temp_poly);
      return(TermList(lead));
    }
    else return(TermList::zero_termlist);
  }
}

TermList operator % (const TermList& tl, const Polynomial& poly) {
  
  TermNode *temp, *lead, *curr;

  if (is_zeropoly(poly)) {
    cout << "Error in % (TermList,Polynomial) - divide by zero\n";
    return(TermList::zero_termlist);
  }
  Polynomial temp_poly;
  temp = tl.tnode;
  lead = 0;
  while (temp != 0) {
    temp_poly = temp->coeff_poly() % poly;
    if (!is_zeropoly(temp_poly)) {
      if (lead == 0) {
	lead = new TermNode(temp->degree_of(),temp_poly);
	curr = lead; 
      }
      else {
	curr->set_next(TermNode(temp->degree_of(),temp_poly));
	curr = curr->next_of();
      }
    }
    temp = temp->next_of();
  }
  if (lead == 0) return(TermList::zero_termlist);
  else return(TermList(lead));
}

TermList operator % (const TermList& tl, const Field_element& fe) {
  
  TermNode *temp, *lead, *curr;

  if (is_zero(fe)) {
    cout << "Error in % (TermList,Field_element) - divide by zero\n";
    return(TermList::zero_termlist);
  }
  Polynomial temp_poly;
  temp = tl.tnode;
  lead = 0;
  while (temp != 0) {
    temp_poly = temp->coeff_poly() % fe;
    if (!is_zeropoly(temp_poly)) {
      if (lead == 0) {
	lead = new TermNode(temp->degree_of(),temp_poly);
	curr = lead; 
      }
      else {
	curr->set_next(TermNode(temp->degree_of(),temp_poly));
	curr = curr->next_of();
      }
    }
    temp = temp->next_of();
  }
  if (lead == 0) return(TermList::zero_termlist);
  else return(TermList(lead));
}

ostream&
operator << (ostream& os, const TermList& tl) {

  TermNode* temp;
  temp = tl.tnode;
  while (temp != 0) {
    os << "\n";
    os << temp->degree_of() << " ";
    os << temp->coeff_poly();
    temp = temp->next_of();
  }
}

void
TermList::print(ostream& os, int disp) {
  TermNode* temp;
  temp = tnode;
  while (temp != 0) {
    os << "\n";
    for (int i=0;i < disp; i++)
      os << " ";
    os << temp->degree_of() << " ";
    temp->coeff_poly().print(os,disp+1);
    temp = temp->next_of();
  }
}

