
extern int Debug; 
#include "triangulate.h"
#include "polynomial.h"

/* Implements a naive triangulation algorithm for the given list of
   polynomials. */
List<Polynomial>
triangulate (List<Polynomial> given_list) 
{
   if (given_list.is_empty())  // No elements in list or 1 element. 
     return(given_list); 
   else if (given_list.size_of() == 1) 
     return(given_list); 

   List<Polynomial> result, result2, temp_list, local_list; 
   ListNode<Polynomial>* min_polyp; 

   local_list = copy_list(given_list); // copies pointers only
   temp_list = find_highest(local_list); 
		 // NOTE : removes the polys found 

   if (temp_list.size_of() == 1) {
       result = triangulate(local_list); 
       result.prepend(temp_list.front_of()->item_of()); 
   }
   else { // more than one poly in variable
       result2 = reduce(temp_list);  // reduce returns min poly as first elt
       min_polyp = result2.delete_head(); 
       result = concat(local_list,result2);  
       result = triangulate(result); 
       result.prepend(min_polyp->item_of()); 
   }
   return result;
}

/* Find the polynomials with highest variable (i.e. with lowest index)
   and _removes_ them from the current list and stores them in the result */
List<Polynomial>
find_highest(List<Polynomial> &given_list)
{
  if (given_list.is_empty()) { // No elements in list or 1 element. 
     return(given_list); 
  }

  List<Polynomial> result; 
  List<Polynomial> rest_list; 
  ListNode<Polynomial>* pnode_p; 
  Polynomial curr_poly; 
  Variable* var_ptr; 
  int low_index; 
  int temp_index; 
  int cur_degree,done; 

  low_index = max_index(given_list); 
  if (low_index == -1) { // means all polynomials were terminal polys
    return(given_list); 
  }

  for (pnode_p=given_list.front_of();pnode_p!=0;pnode_p = pnode_p->next_of()) {
      curr_poly = pnode_p->item_of();  
      if (!is_terminal(curr_poly)) { 
        var_ptr = variable_of(curr_poly); 
	cur_degree = degree(curr_poly); 
	temp_index = var_ptr->index(); 
	done = 0; 
	while ((cur_degree == 0) && (!done)) { 
	  curr_poly = lcoeff(curr_poly); 
	  if (is_terminal(curr_poly))
	    done = 1; 
	  else {
	    var_ptr = variable_of(curr_poly); 
	    cur_degree = degree(curr_poly); 
	    temp_index = var_ptr->index(); 
	  }
	}
	if (temp_index == low_index) {
	  result.append(curr_poly); 
	}
	else rest_list.append(curr_poly); 
      }
  }
  given_list = copy_list(rest_list); 
  return result; 
}

int
max_index(List<Polynomial> list) 
{ 
  ListNode<Polynomial>* pnode_p; 
  Polynomial curr_poly; 
  Variable* var_ptr; 
  int ind_result = -1; 
  int done; 
  int index, low_index,cur_degree; 
  for (pnode_p=list.front_of();pnode_p!=0;pnode_p=pnode_p->next_of()) {
      curr_poly = pnode_p->item_of();  
      if (!is_terminal(curr_poly)) { 
        var_ptr = variable_of(curr_poly); 
	cur_degree = degree(curr_poly); 
	index = var_ptr->index(); 
	done = 0; 
	while ((cur_degree == 0) && (!done)) {
	  curr_poly = lcoeff(curr_poly); 
	  if (is_terminal(curr_poly))
	    done = 1; 
	  else {
	    var_ptr = variable_of(curr_poly); 
	    cur_degree = degree(curr_poly); 
	    index = var_ptr->index(); 
	  }
	} 
	if (index != -1) { 
	  if (ind_result > -1) {
	    if (index < ind_result) {
	      if (degree > 0) 
	      ind_result = index; 
	    }
	  }
	  else {
	    ind_result = index; 
	  }
	}
      } 
  }
  low_index = ind_result; 
  return low_index; 
}

/* Assumes all polynomials have the same leading variable. 
   Attempts to find a polynomial of degree 1 first and reduces
   all the others w.r.t. it. If no such polynomial exists then
   attempts to find the lowest degree one and reduces w.r.t that
   polynomial.   

   At the end of process, it will return a list whose first element
   is the poly which is the final reducing poly i.e. the one which
   will be designated f_i for the ith step in the triangulation. 
   The rest of the list will be the polynomials without x_i
*/

List<Polynomial>
reduce(List<Polynomial> list) 
{
  List<Polynomial> result;
  List<Polynomial> result2;
  List<Polynomial> local_list;
  ListNode<Polynomial>* pnode_p; 
  Polynomial temp; 
  Polynomial* poly_ptr; 
  int min_degree = -1; 
  int cur_degree,done; 


  if (list.is_empty()) 
    return(list); 
  if (list.size_of() == 1) 
    return(list); 

  ListNode<Polynomial>* min_node; 
  min_node = list.front_of();
  min_degree = degree(min_node->item_of());
  for (pnode_p=list.front_of();pnode_p!=0;pnode_p=pnode_p->next_of()) {
      temp = pnode_p->item_of();  
      cur_degree = degree(temp); 
      done = 0; 
      while ((cur_degree == 0) && (!done)) {
	  temp = lcoeff(temp); 
	  if (is_terminal(temp))
	    done = 1; 
	  else {
	    cur_degree = degree(temp); 
	  }
      } 
      if (cur_degree < min_degree) {
	min_degree = degree(temp); 
	min_node = pnode_p; 
      }
  } 
  Polynomial min_poly = min_node->item_of(); 
  for (pnode_p=list.front_of();pnode_p!=0;pnode_p=pnode_p->next_of()) {
      temp = pnode_p->item_of(); 
      if (pnode_p != min_node) { 
	if (Debug) { 
	  cout << temp << "\n"; 
	  cout << min_poly << "\n"; 
	} 
	temp = prem(temp,min_poly);
	if (Debug) { 
	  cout << temp << "\n"; 
	} 
	if (is_zeropoly(temp)) 
	  cout << "Warning - zero polynomial in triangulation\n"; 
        result.append(temp); 
      }
  }
  result.prepend(min_poly); 
  if (min_degree == 1) {
    return result; 
  } 

  local_list = find_highest(result);
  if (local_list.is_empty()) { 
      cout << "Error in reduce() - null local list\n"; 
  }
  else if (local_list.size_of() == 1) {
      result2 = local_list; 
      result = concat(result2,result); 
  }
  else { 
      result2 = reduce(local_list); 
      result = concat(result2,result); 
  }
  return result; 
}
