
/*************************************************************************

  APU R 2.0

  slp_bigint.c

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  
#include "field.h"
#include "slp.h"

Slp
operator + (const Slp& a, const BigInt& b) {
  if (is_zero(a)) return(Slp(b)); 
  else if (is_zero(b)) return(a);
  else return(slp_binop(ADD,a,Slp(b)));
}

Slp
operator + (const BigInt& a, const Slp& b) {
  if (is_zero(a)) return(b); 
  else if (is_zero(b)) return(Slp(a));
  else return(slp_binop(ADD,Slp(a),b));
}

Slp
operator - (const BigInt& a, const Slp&b) {
  if (is_zero(a)) return(-b); 
  else if (is_zero(b)) return(Slp(a));
  return(slp_binop(SUB,Slp(a),b));
}

Slp
operator - (const Slp& a, const BigInt&b) {
  if (is_zero(a)) return(Slp(-b)); 
  else if (is_zero(b)) return(a);
  return(slp_binop(SUB,a,Slp(b)));
}

Slp
operator * (const BigInt& a, const Slp&b) {
  if (is_zero(a) || is_zero(b)) return(Slp::zero_slp); 
  else return(slp_binop(MULT,Slp(a),b));
}

Slp
operator * (const Slp& a, const BigInt&b) {
  if (is_zero(a) || is_zero(b)) return(Slp::zero_slp); 
  else return(slp_binop(MULT,a,Slp(b)));
}

Slp
operator / (const BigInt& a, const Slp& b) {
  if (is_zero(a)) return(Slp::zero_slp);
  else if (is_zero(b)) 
    cout << "Divide by zero_slp in / (BigInt,Slp) \n";
  else return(slp_binop(DIV,Slp(a),b));
}

Slp
operator / (const Slp& a, const BigInt& b) {
  if (is_zero(a)) return(Slp::zero_slp);
  else if (is_zero(b)) 
    cout << "Divide by zero in / (Slp,BigInt) \n";
  else return(slp_binop(DIV,a,Slp(b)));
}

