
/*************************************************************************

  APU R 2.0

  slp_eval2.c

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  
#include "field.h"
#include "slp.h"
#include "polynomial.h"
#include "multimod_arith.h"
extern int Debug; 

Field_element eval_multimodp(const Slp& slp,char* var,const Field_element& fe, int* non_sing);
Field_element eval_multimodp2(const Slp& slp,int* non_sing=0);
void eval_multimodp_mem(Slp* slp,char* var,int* fe, int* non_sing, 
			int* result);

/* Returns a MultiIntModp type. For description of inputs see comments 
   for evaluate_modp(). The parameter "mem" chooses whether the
   memory efficient version is used or not. 
*/

Field_element
evaluate_multimodp(const Slp& slp, char* diff_var = "", int* non_sing=0, 
                   char* var = "",Field_element fe =0, int mem=1)
{
  if (strcmp(var,"") && (non_sing != 0)) { 
    if (mem) { 
      int result_arr[MULTI_NUMPRIMES];
      eval_multimodp_mem(&slp,var,frep2_of(fe),non_sing,result_arr);
      return(Field_element(result_arr));
    }
    else {
      Field_element result = eval_multimodp(slp,var,fe,non_sing); 
      return result; 
    }
  }
  else if (!strcmp(var,"")) { // Purely numerical evaluation. 
    Field_element result = eval_multimodp2(slp,non_sing); 
    if (Debug) { 
      cout << result.type_of() << "\n"; 
      cout << result << "\n"; 
    }
    return result;
  }
  else cout << "Can't handle this case in evaluate_multimodp() currently\n"; 
}

Field_element
eval_multimodp(const Slp& slp,char* var,const Field_element& fe, int* non_sing)
{
  if (is_zero(slp)) 
    return(make_Field_element(0,MULTI_INTMODP));
  if (eval_level(slp) == Slp::Eval_level) {
    return(value(slp)); 
  }
  int slp_op = op(slp); 
  if (slp_op == NOP_FE) {
    if (type_of(value(slp)) == MULTI_INTMODP)  
      return(value(slp)); 
    if (type_of(value(slp)) == SMALLINT)  
      return(make_Field_element((value(slp)).i_value(),MULTI_INTMODP)); 
    else cout << "Error in eval_multimodp - unknown numerical node type\n";
  }
  else if (slp_op == NOP_VAR) {
    if (!strcmp(var,name(variable_of(slp)))) { 
      return(fe);
    }
    else {
      // return(value(slp)); 
      if (type_of(value(slp)) == SMALLINT)  
        return(make_Field_element((value(slp)).i_value(),MULTI_INTMODP)); 
      else return(value(slp)); 
    }
  }
  /*
  else if (slp_op == DIFF) {
     "Error in eval_multimodp(Slp) - DIFF op cannot be handled\n";
  }
  */
  else {
    Field_element result, lval, rval;
    int l_non_sing = 1, r_non_sing = 1; 
    lval = eval_multimodp(lchild(slp),var,fe,&l_non_sing);  
    if (slp_op != EXP) 
        rval = eval_multimodp(rchild(slp),var,fe,&r_non_sing);
    else r_non_sing = 1; 
    if (l_non_sing && r_non_sing) 
	*non_sing = 1; 
    else *non_sing = 0; 
    if (*non_sing == 0) { 
      result = make_Field_element(0,MULTI_INTMODP); 
      slp.set_value(result);
      slp.eval_level(Slp::Eval_level);
      return(result); 
    } 
    switch(slp_op) {
      case ADD : 
	result = lval+rval; break;
      case SUB : 
	result = lval-rval; break;
      case MULT : 
	// slp.set_value(lval*rval);
        // slp.eval_level(Slp::Eval_level);
	// return(value(slp));
	result = lval * rval; break;
      case DIV : 
	if (is_zero(rval)) {
	    *non_sing = 0; 
	    result = make_Field_element(0,MULTI_INTMODP);
	}
	else {
	    *non_sing = 1; 
	    result = lval/rval; 
	}
        break;
      case EXP : 
	/*
	if (op(rchild(slp)) != NOP_FE) 
	  cout << "Error in Slp.eval_multimodp()\n";
        else {
	*/
	  int temp;
	  rval = value(rchild(slp)); 
	  /*
	  if (type_of(rval) != SMALLINT) 
	    cout << "Error in Slp.eval_multimodp()\n";
	  else 
	  */
	  temp = ((SmallInt&)(rval)).i_value(); 
	  result = lval ^ temp;
	  /*
	}
	*/
	break;
      default : cout << "Error in Slp.eval_multimodp() - unknown operator\n"; 
    }
    slp.set_value(result);
    slp.eval_level(Slp::Eval_level);
    return(result);
  }
}

Field_element
eval_multimodp2(const Slp& slp,int* non_sing=0)
{
  if (is_zero(slp)) 
    return(make_Field_element(0,MULTI_INTMODP));
  if (eval_level(slp) == Slp::Eval_level) {
    return(value(slp)); 
  }
  int slp_op = op(slp); 
  if (slp_op == NOP_FE) {
    if (type_of(value(slp)) == MULTI_INTMODP)  
      return(value(slp)); 
    if (type_of(value(slp)) == SMALLINT)  
      return(make_Field_element((value(slp)).i_value(),MULTI_INTMODP)); 
    else if (type_of(value(slp)) == BIGINT)
      return(MultiIntModp(value(slp)));
    else cout << "Error in eval_multimodp - unknown numerical node type\n";
  }
  else if (slp_op == NOP_VAR) {
      // return(value(slp)); 
      if (type_of(value(slp)) == SMALLINT)  
        return(make_Field_element((value(slp)).i_value(),MULTI_INTMODP)); 
      else if (type_of(value(slp)) == BIGINT)
        return(MultiIntModp(value(slp))); 
      else return(value(slp)); 
  }
  else {
    Field_element result, lval, rval;
    if (non_sing != 0) { 
      int l_non_sing = 1, r_non_sing = 1; 
      lval = eval_multimodp2(lchild(slp),&l_non_sing);  
      if (slp_op != EXP) 
        rval = eval_multimodp2(rchild(slp),&r_non_sing);
      else r_non_sing = 1; 
      if (l_non_sing && r_non_sing) 
	*non_sing = 1; 
      else *non_sing = 0; 
      if (*non_sing == 0) { 
        result = make_Field_element(0,MULTI_INTMODP); 
        slp.set_value(result);
        slp.eval_level(Slp::Eval_level);
        return(result); 
      } 
    }
    else { 
      lval = eval_multimodp2(lchild(slp)); 
      if (slp_op != EXP) 
	rval = eval_multimodp2(rchild(slp)); 
      if (Debug) { 
	cout << lval << "\n"; 
	cout << rval << "\n"; 
      }
    }
    switch(slp_op) {
      case ADD : 
	result = lval+rval; break;
      case SUB : 
	result = lval-rval; break;
      case MULT : 
	result = lval * rval; break;
      case DIV : 
	if (non_sing != 0) {
	  if (is_zero(rval)) {
	    *non_sing = 0; 
	    result = make_Field_element(0,MULTI_INTMODP);
	  }
	  else {
	    *non_sing = 1; 
	    result = lval/rval; 
	  }
	}
	else result = lval/rval; 
        break;
      case EXP : 
	/*
	if (op(rchild(slp)) != NOP_FE) 
	  cout << "Error in Slp.eval_multimodp()\n";
        else {
	*/
	  int temp;
	  rval = value(rchild(slp)); 
	  /*
	  if (type_of(rval) != SMALLINT) 
	    cout << "Error in Slp.eval_multimodp()\n";
	  else 
	  */
	  temp = ((SmallInt&)(rval)).i_value(); 
	  result = lval ^ temp;
	  /*
	}
	*/
	break;
      default : cout << "Error in Slp.eval_multimodp() - unknown operator\n"; 
    }
    slp.set_value(result);
    slp.eval_level(Slp::Eval_level);
    if (Debug) { 
      cout << result.type_of() << "\n"; 
      cout << result << "\n"; 
    }
    return(result);
  }
}

/* Attempt at a more efficient multimodp eval. The problem with the 
   earlier version is that the use of operators for field elements, 
   and the returning of a MultiIntModp type causes copying of 
   array elements individually instead of as a pointer. In this routine,
   the result is allocated in the array "result" and passed to the 
   evaluation routine. This eliminates multiple copying of the arrays 
   (as arrays instead of as pointers) */
void
eval_multimodp_mem(Slp* slp,char* var,int* fe, int* non_sing, int* result)
{
  if (is_zero(*slp)) {
    multimod_make_zeroarray(result);
    return;
  }
  if (eval_level(*slp) == Slp::Eval_level) {
    multimod_copy_array(result,frep2_of(*(value_ptr(*slp))));
    return;
  }
  int slp_op = op(*slp); 
  if (slp_op == NOP_FE) {
    if (type_of(*(value_ptr(*slp))) == MULTI_INTMODP) {
      multimod_copy_array(result,frep2_of(*(value_ptr(*slp))));
      return;
    }
    if (type_of(*(value_ptr(*slp))) == SMALLINT) {
      multimod_set_array(result,(*(value_ptr(*slp))).i_value());
      return;
    }
    else cout << "Error in eval_multimodp - unknown numerical node type\n";
  }
  else if (slp_op == NOP_VAR) {
    if (!strcmp(var,name(variable_of(*slp)))) { 
      multimod_copy_array(result,fe);
      return;
    }
    else {
      // return(value(slp)); 
      if (type_of(*(value_ptr(*slp))) == SMALLINT) {
	multimod_set_array(result,(*(value_ptr(*slp))).i_value());
	return;
      }
      else { 
	multimod_set_array(result,(*(value_ptr(*slp))).i_value());
	return;
      }
    }
  }
  /* For now, comment out to reduce tests
  else if (slp_op == DIFF) {
     "Error in eval_multimodp(Slp) - DIFF op cannot be handled\n";
  }
  */
  else {
    int lval[EXTRA_ENTRIES]; int rval[EXTRA_ENTRIES];
    int l_non_sing = 1, r_non_sing = 1; 
    eval_multimodp_mem(((Slp_node*)(slp->frep))->left,var,fe,&l_non_sing,lval);  
    if (slp_op != EXP) 
        eval_multimodp_mem(((Slp_node*)(slp->frep))->right,var,fe,&r_non_sing,rval);
    else r_non_sing = 1; 
    if (l_non_sing && r_non_sing) 
	*non_sing = 1; 
    else *non_sing = 0; 
    if (*non_sing == 0) { 
      multimod_make_zeroarray(result);
      (*slp).set_value(Field_element(result));
      (*slp).eval_level(Slp::Eval_level);
      return; 
    } 
    int temp; Field_element temp_fe;
    switch(slp_op) {
      case ADD : 
	multimod_add(result,lval,rval); break;
      case SUB : 
	multimod_subtract(result,lval,rval); break;
      case MULT : 
	multimod_multiply(result,lval,rval); break;
      case DIV : 
	if (is_zeroarray(rval)) {
	    *non_sing = 0; 
	    multimod_make_zeroarray(result);
	    return;
	}
	else {
	    *non_sing = 1; 
	    multimod_divide(result,lval,rval); 
	}
        break;
      case EXP : 
	temp_fe = *(value_ptr(rchild(*slp)));
	temp = ((SmallInt&)(temp_fe)).i_value(); 
	multimod_exp(result,lval,temp);
	break;
      default : cout << "Error in Slp.eval_multimodp_mem() - unknown operator\n"; 
    }
    (*slp).set_value(Field_element(result));
    (*slp).eval_level(Slp::Eval_level);
  }
}

