
/*************************************************************************

  APU R 2.0

  ludecomp.c

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  
#include "field.h"
#include "matrix.h"
#include "slp.h"

/* Given an n x n Slp matrix mat, ludecomp() _replaces_ it by the LU 
   decomposition of a rowwise permutation of mat. 
   NOTE THAT THIS ROUTINE WILL CHANGE THE ORIGINAL MATRIX. 
*/

int 
ludecomp(Matrix<Slp> mat)
{
  int i,j,k; 
  int sign = 1; 
  int n = dim1_of(mat); 
  Slp sum; 
  Slp::Degree_eval_level++; 
  for (j=0;j<n;j++) { 
    for (i=0;i<j;i++) { 
      sum = mat[i][j]; 
      for (k=0;k<i;k++) 
	sum = sum - mat[i][k]*mat[k][j]; 
      mat[i][j] = sum; 
    }
    for (i=j;i<n;i++) { 
      sum = mat[i][j]; 
      for (k=0;k<j;k++) 
	sum = sum - mat[i][k]*mat[k][j]; 
      mat[i][j] = sum;
    }
    int p = j; 
    while((p<n) && (is_zero(evaluate_modp(mat[p][j])))) { 
      p++; 
    }
    if (p < n) { 
      if (p!=j) { 
	for (k=0;k<n;k++) { 
	  Slp temp = mat[p][k]; 
	  mat[p][k] = mat[j][k]; 
	  mat[j][k] = temp; 
	}
	sign = -sign; 
      }
      if (j!=n-1) { 
	for (i=j+1;i<n;i++) 
	  mat[i][j] = mat[i][j]/mat[j][j]; 
      }
    }
    else { 
      cout << "Couldn't find pivot\n"; 
    }
  }
  return sign; 
}

/* Computes LU decomposition and returns determinant of the matrix mat. 
   NOTE THAT THIS ROUTINE WILL CHANGE THE ORIGINAL MATRIX. 
*/
Slp 
ludecomp_det(Matrix<Slp> mat) 
{ 
  int sign = ludecomp(mat); 
  Slp result = Slp(sign); 
  int n = dim1_of(mat); 
  for (int i=0;i<n;i++) 
    result = result*mat[i][i]; 
  return result; 
}
