
/*************************************************************************

  APU R 2.0

  slp_symderiv.c

  (c) 1996, Regents of the University of California. All rights reserved. 

  Author : Ashutosh Rege (rege@cs.berkeley.edu)

*************************************************************************/
  
#include "stddefs.h"
#include "slp.h"
#include "array.h"
#include "slpsymder.h"
#include "slpder.h"

Slp
symbolic_slp_partial_derivative(Slp slp)
{
  Slp original,result;
  int der_power;

  der_power = 1;
  result = sym_slp_partial_derivative2(slp,der_power);
  return(result);
}

Slp
sym_slp_partial_derivative2(Slp slp, int power)
{
  Slp result;
  int der_computed;

  result = sym_find_partial_derivative(slp,power,&der_computed);
  if (!der_computed) {
    result = sym_compute_slp_partial_derivative(slp,power);
  }
  return(result);
}

Slp
sym_find_partial_derivative(Slp slp, int power, int *deriv_computed)
{
   Slp result;
   if (power == 0) {
     result = slp;
     *deriv_computed = 1;
   }
   else if (is_empty(deriv_array(slp))) {
     result = Slp::zero_slp;
     *deriv_computed = 0;
   }
   else if (is_empty(deriv_computed_array(slp))) {
     result = Slp::zero_slp;
     *deriv_computed = 0;
   }
   else if (deriv_computed_array(slp)[0] != -1) {
     result = Slp::zero_slp;
     *deriv_computed = 0;
   }
   else {
     result = sym_find_partial_derivative(deriv_array(slp)[0],power-1,
				      deriv_computed);
   }
   return(result);
}


/* computes the derivative slp of the given slp wrt the given variable,
   i.e. the derivative has to be computed wrt to one variable only */

Slp
sym_compute_slp_single_derivative(Slp slp)
{
  Slp l_der, r_der,result,temp_slp;
  Slp tempres;
  int i,j,done,curr_index;
  int coeff,coeff1,coeff2;
  int singleder,singlevar;

  if (is_zero(slp)) 
    return(Slp::zero_slp);

  if ((op(slp) == ADD) || (op(slp) == SUB)) {
      l_der = symbolic_slp_partial_derivative(lchild(slp));
      r_der = symbolic_slp_partial_derivative(rchild(slp));
      if (op(slp) == ADD) 
	result = l_der + r_der; 
      else if (op(slp) == SUB) 
	result = l_der - r_der; 
      if (is_empty(deriv_array(slp))) {
	deriv_array(slp) = Array<Slp>(1);
	deriv_computed_array(slp) = Array<int>(1);
      }
      deriv_array(slp)[0] = result;
      deriv_computed_array(slp)[0] = -1;
      if (!is_zero(result))
        anti_deriv(result) = slp;
  }
  else if (op(slp) == MULT) {

        l_der = symbolic_slp_partial_derivative(lchild(slp));
        r_der = symbolic_slp_partial_derivative(rchild(slp));
	result = r_der * lchild(slp));
        tempres = l_der * rchild(slp));
	result = result + tempres);
        if (zero_slp(deriv_array(slp))) {
	  deriv_array(slp) = Array<Slp>(1);
	  deriv_computed_array(slp) = Array<int>(1);
        }
	deriv_array(slp)[0] = result;
        deriv_computed_array(slp)[0] = -1;
        if (result != ZERO_SLP) 
	  anti_deriv(result) = slp;
  }
  else if (op(slp) == DIV) {

    l_der = symbolic_slp_partial_derivative(lchild(slp));
    r_der = symbolic_slp_partial_derivative(rchild(slp));
    tempres = slp_multiply(rchild(slp),l_der);
    result = slp_multiply(lchild(slp),r_der);
    result = slp_subtract(tempres,result,NULL);
    tempres = slp_multiply(rchild(slp),rchild(slp));
    result = slp_divide(result,tempres);
    if (zero_slp(deriv_array(slp))) {
	  deriv_array(slp) = Array<Slp>(1);
	  deriv_computed_array(slp) = Array<int>(1);
    }
    deriv_array(slp)[0] = result;
    deriv_computed_array(slp)[0] = -1;
    if (result != ZERO_SLP) 
      anti_deriv(result) = slp;

  }
  else if ((op(slp) == NOP) || (op(slp) == DIFF))  {
    result = sym_slp_single_terminal_derivative(slp);
  }
  else 
    printf("Error - SLP node with no legal operator\n");

  if (!zero_slp(result))
   if ((result->index >= Cur_free_slp) || (result->free == 1)) 
    printf("ERROR\n");
  return(result);

}

Slp
sym_slp_single_terminal_derivative(Slp slp)
{
  Slp result;
  Field_element fe_temp;

  if (is_slpnumnode(slp)) {
    result = ZERO_SLP;
  }
  else {
    result = make_diff_slp(slp);
  }
  if (zero_slp(deriv_array(slp))) {
    deriv_array(slp) = Array<Slp>(1);
    deriv_computed_array(slp) = Array<int>(1);
  }
  deriv_array(slp)[0] = result;
  deriv_computed_array(slp)[0] = -1;
  if (!zero_slp(result)) 
    if ((result->index >= Cur_free_slp) || (result->free == 1))
      printf("Error in sym_slp_std\n");
  return(result);
}

Slp
sym_compute_slp_partial_derivative(Slp slp, int der_power)
{
  Field_element temp_fe;
  Slp lder,rder,temp_slp,temp_result,result;
  int lpower,rpower,coeff;

  if ((slp->index > Cur_free_slp) || (slp->free == 1))
    printf("EERROR\n");
  if (der_power == 0) {
    result = slp;
  }
  else if (der_power = 1) {
    result = sym_compute_slp_single_derivative(slp);
  }
  else {
    if (slp == ZERO_SLP) {
    }
    else if ((op(slp) == ADD) || (op(slp) == SUB)) {
      lder = sym_slp_partial_derivative2(lchild(slp),der_power);
      rder = sym_slp_partial_derivative2(rchild(slp),der_power);
      if (op(slp) == ADD) {
	result = slp_add(lder,rder);
      }
      else if (op(slp) == SUB) {
	result = slp_subtract(lder,rder);
      }
      if (deriv_array(slp) == NULL) {
	deriv_array(slp) = Array<Slp>(1);
	deriv_computed_array(slp) = Array<int>(1);
      }
      deriv_array(slp)[0] = result;
      deriv_computed_array(slp)[0] = -1;
      if (result != ZERO_SLP) 
        anti_deriv(result) = slp;
    }
    else if (op(slp) == MULT) {
     /* multiplication is handled as follows : repeated terms can
     occur under a straightforward expansion, so doing it in that
     manner will lead to very large SLP's. Instead the expansion
     is done according to the formula for multiplication. e.g.
     if h = f.g and we want to compute the twice partial wrt x,
     then d2h/dx2 = d2f/dx2.g + 2. df/dx . dg/dx + d2g/dx2.f
     which takes up less space.

     The function "update" generates the variables wrt which
     the partials have to be taken.
     */
     lpower = der_power;
     rpower = 0;
     result = ZERO_SLP;
     while (lpower >= 0) {
       lder = sym_slp_partial_derivative2(lchild(slp),lpower);
       rder = sym_slp_partial_derivative2(rchild(slp),rpower);
       temp_result = slp_multiply(lder,rder);
       coeff = binomial_coeff(der_power,lpower);
       if (coeff == 2) {
	 temp_result = slp_add(temp_result,temp_result);
       }
       else if (coeff != 1) {
	 temp_fe = makeset_field_element_long(coeff,SLP_TYPE);
	 temp_slp = make_terminal_slp(temp_fe);
	 temp_result = slp_multiply(temp_result,temp_slp);
       }
       result = slp_add(result,temp_result);
       lpower--;
       rpower++;
     }
     if (deriv_array(slp) == NULL) {
	deriv_array(slp) = Array<Slp>(1);
	deriv_computed_array(slp) = Array<int>(1);
     }
     deriv_array(slp)[0] = result;
     deriv_computed_array(slp)[0] = -1;
     if (result != ZERO_SLP) 
       anti_deriv(result) = slp;
    }
    else if (op(slp) == DIV) {
       lder = sym_slp_partial_derivative2(lchild(slp),der_power);
       temp_result = slp_multiply(lder,rchild(slp));
       rder = sym_slp_partial_derivative2(rchild(slp),der_power);
       result = slp_multiply(rder,lchild(slp));
       result = slp_subtract(temp_result,result);
       temp_result = slp_multiply(rchild(slp),rchild(slp));
       result = slp_divide(result,temp_result);
       if (deriv_array(slp) == NULL) {
	 deriv_array(slp) = Array<Slp>(1);
	 deriv_computed_array(slp) = Array<int>(1);
       }
       deriv_array(slp)[0] = result;
       deriv_computed_array(slp)[0] = -1;
       if (result != ZERO_SLP) 
         anti_deriv(result) = slp;
    }
    else if ((op(slp) == NOP) || (op(slp) == DIFF)) {
       result = ZERO_SLP; /* since der_power > 1 */
    }
    else {
      printf("Error - slp node with invalid operator\n");
    }
  }
  if (!zero_slp(result))
   if ((result->index >= Cur_free_slp) || (result->free == 1)) 
    printf("ERROR\n");
  return(result);
}

